import intake
import argparse
import pkg_resources as pkgr


def parse(cliargs=None):
    """parse the command line arguments"""
    parser = argparse.ArgumentParser(
        description="list observational datasets available"
    )

    parser.add_argument(
        "--platform",
        type=str,
        required=False,
        default="gfdl",
        help="computing platform, default is gfdl",
    )
    cmdLineArgs = parser.parse_args(cliargs)
    return cmdLineArgs


def run(dictArgs):
    cat_platform = "catalogs/obs_catalog_" + dictArgs["platform"] + ".yml"
    if pkgr.resource_exists("omlabs", cat_platform):
        catfile = pkgr.resource_filename("omlabs", cat_platform)
        cat = intake.open_catalog(catfile)
        return cat
    else:
        print("Platform not available")


def parse_and_run(cliargs=None):
    cmdLineArgs = parse(cliargs)
    dictArgs = vars(cmdLineArgs)
    catobs = run(dictArgs)
    if catobs is not None:
        for obs in list(catobs):
            print(f"{obs}: {catobs[obs].description}")


if __name__ == "__main__":
    parse_and_run()



import warnings
import palettable

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

from omlabs import m6plot

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")


def plot_z(otsfn, lat, depth, label=None, dates=None):
    """Function to plot 3-panel MOC figure

    Parameters
    ----------
    otsfn : numpy.ma.MaskedArray
        Input array with dimensions (basin,depth,lat)
    lat : numpy.array
        1-D array of latitude values
    depth : numpy.array
        1-D array of depth values
    label : str, optional
        Experiment name / top line label, by default None
    dates : tuple, optional
        Tuple of date ranges, by default None

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """

    # Set up the figure
    fig = plt.figure(figsize=(8.5, 11))

    # Top panel: Atlantic-Arctic
    arr = otsfn[0]
    ax1 = plt.subplot(3, 1, 1, facecolor="#bbbbbb")
    cb = plot_z_panel(ax1, arr, lat, depth, xlim=(-33, 90))
    ax1.text(
        0.01,
        1.02,
        "a. Atlantic-Arctic",
        ha="left",
        fontsize=12,
        transform=ax1.transAxes,
    )

    # Basin maximum
    basinmax = z_extremes(arr, lat, depth, ylim=(-33, 90), zlim=(500, 2000))
    maxval = str(round(basinmax[2], 1))
    ax1.plot(*basinmax[0:2], marker="o", markersize=5, color="w")
    ax1.text(*basinmax[0:2], maxval, fontsize=16, ha="right", va="top")
    ax1.plot(60, 4500, marker="o", markersize=5, color="w")
    ax1.text(63, 4500, "N. Atl. Max", fontsize=10, ha="left", va="center")

    # Maximum at latitude of RAPID array
    rapid = z_extremes(arr, lat, depth, ylim=(25, 28.0), zlim=(500, 2000))
    maxval = str(round(rapid[2], 1))
    ax1.plot(*rapid[0:2], marker="o", markersize=5, color="#ffff00")
    ax1.text(*rapid[0:2], maxval, fontsize=16, ha="left", va="bottom")
    ax1.plot(60, 5500, marker="o", markersize=5, color="#ffff00")
    ax1.text(63, 5500, "RAPID", fontsize=10, ha="left", va="center")

    # Middle panel: Indo-Pacific
    arr = otsfn[1]
    ax2 = plt.subplot(3, 1, 2, facecolor="#bbbbbb")
    _ = plot_z_panel(ax2, arr, lat, depth, xlim=(-40, 65))
    ax2.text(
        0.01, 1.02, "b. Indo-Pacific", ha="left", fontsize=12, transform=ax2.transAxes
    )

    # Bottom panel: Global
    arr = otsfn[2]
    ax3 = plt.subplot(3, 1, 3, facecolor="#bbbbbb")
    _ = plot_z_panel(ax3, arr, lat, depth)
    ax3.text(0.01, 1.02, "c. Global", ha="left", fontsize=12, transform=ax3.transAxes)

    # Add dates to top panel
    if dates is not None:
        dates = f"Years {dates[0]} - {dates[1]}"
        ax1.text(0.98, 1.02, dates, ha="right", fontsize=12, transform=ax1.transAxes)

    # adjust panel spacing
    plt.subplots_adjust(hspace=0.25)

    # add top label
    plt.suptitle(label)

    # colorbar
    cbar_ax = fig.add_axes([0.30, 0.05, 0.4, 0.01])
    fig.colorbar(cb, cax=cbar_ax, orientation="horizontal")

    return fig


def plot_z_panel(ax, arr, lat, depth, levels=None, xlim=None):
    """Plots an indiviual MOC panel

    Parameters
    ----------
    ax : matplotlib.axes.axis
        Axis handle
    arr : numpy.ma.MaskedArray
        2-D masked array with dimensions (depth,lat)
    lat : numpy.array
        1-D array of latitude values
    depth : numpy.array
        1-D array of depth values
    levels : list, optional
        list of contour intervals, by default None
    xlim : tuple, optional
        latitude range for plotting, by default None

    Returns
    -------
    matplotlib.collections.QuadMesh
        output from pcolormesh
    """

    levels = np.arange(-45, 48, 3) if levels is None else levels
    cb = ax.contourf(lat, depth, arr, levels=levels, cmap="RdBu_r")
    ax.contour(lat, depth, arr, levels=levels, colors=["k"], linewidths=0.5)
    ax.contour(lat, depth, arr, levels=[0.0], colors=["k"], linewidths=1)
    ax.set_yscale("splitscale", zval=[6500.0, 2000.0, 0.0])
    ax.set_xlim(xlim)

    return cb


def z_extremes(arr, lat, depth, zlim=None, ylim=None):
    """Locate max value of a lat-depth array

    Parameters
    ----------
    arr : numpy.ma.MaskedArray
        2-D masked array with dimensions (depth,lat)
    lat : numpy.array
        1-D array of latitude values
    depth : numpy.array
        1-D array of depth values
    zlim : tuple, optional
        depth range, by default None
    ylim : tuple, optional
        latitude range, by default None

    Returns
    -------
    tuple
        (lat,depth,max. value)
    """

    # broadcast lat and depth to 2D arrays
    lat = np.tile(lat[None, :], (arr.shape[0], 1))
    depth = np.tile(depth[:, None], (1, arr.shape[1]))

    # latitude range mask
    if ylim is not None:
        assert isinstance(ylim, tuple), "ylim must be a tuple"
        ymask = np.where(lat >= ylim[0], 1.0, 0.0)
        ymask = np.where(lat <= ylim[1], ymask, 0.0)
    else:
        ymask = 1.0

    # depth range mask
    if zlim is not None:
        assert isinstance(zlim, tuple), "zlim must be a tuple"
        zmask = np.where(depth >= zlim[0], 1.0, 0.0)
        zmask = np.where(depth <= zlim[1], zmask, 0.0)
    else:
        zmask = 1.0

    # combine masks
    mask = np.array(ymask * zmask)

    # turn mask into a masked array
    if len(mask.shape) > 0:
        mask = np.ma.masked_where(mask == 0.0, mask)

    # multiply array by mask
    arr = arr * mask

    # find indicies of max values
    ind = np.unravel_index(np.ma.argmax(arr, axis=None), arr.shape)

    return (lat[ind], depth[ind], arr[ind])



import matplotlib.pyplot as plt


def setFigureSize(
    aspect=None,
    verticalresolution=None,
    horiztonalresolution=None,
    npanels=1,
    debug=False,
):
    """
    Set the figure size based on vertical resolution and aspect ratio (tuple of W,H).
    """
    if (not horiztonalresolution is None) and (not verticalresolution is None):
        if aspect is None:
            aspect = [horiztonalresolution, verticalresolution]
        else:
            raise Exception(
                "Aspect-ratio and both h-/v- resolutions can not be specified together"
            )
    if aspect is None:
        aspect = {1: [16, 9], 2: [1, 1], 3: [7, 10]}[npanels]
    if (not horiztonalresolution is None) and (verticalresolution is None):
        verticalresolution = int(1.0 * aspect[1] / aspect[0] * horiztonalresolution)
    if verticalresolution is None:
        verticalresolution = {1: 576, 2: 720, 3: 1200}[npanels]
    width = int(1.0 * aspect[0] / aspect[1] * verticalresolution)  # First guess
    if debug:
        print("setFigureSize: first guess width =", width)
    width = width + (width % 2)  # Make even
    if debug:
        print("setFigureSize: corrected width =", width)
    if debug:
        print("setFigureSize: height =", verticalresolution)
    plt.figure(figsize=(width / 100.0, verticalresolution / 100.0))  # 100 dpi always?
    if npanels == 1:
        plt.gcf().subplots_adjust(
            left=0.08, right=0.99, wspace=0, bottom=0.09, top=0.9, hspace=0
        )
    elif npanels == 2:
        plt.gcf().subplots_adjust(
            left=0.11, right=0.94, wspace=0, bottom=0.09, top=0.9, hspace=0.15
        )
    elif npanels == 3:
        plt.gcf().subplots_adjust(
            left=0.11, right=0.94, wspace=0, bottom=0.05, top=0.93, hspace=0.15
        )
    elif npanels == 0:
        pass
    else:
        raise Exception("npanels out of range")



# -*- coding: utf-8 -*-
"""m6plot - a collection of routines for plotting MOM6 output

This module contains a collection of utilities to create lightweight
plots of MOM6 output. Plots include 2-dimensional maps as well as 
3-dimenstional sections.
"""

__version__ = "0.0.0"
funcs = [
    "addInteractiveCallbacks",
    "addStatusBar",
    "cm",
    "xyplot",
    "xycompare",
    "yzplot",
    #'yzcompare',
    #'ztplot',
    #'brownblue_cmap',
    #'cmoceanRegisterColormaps',
    #'createTZlabels',
    #'createYZlabels',
    #'drawNinoBoxes',
    #'linCI',
    #'newLims',
    #'parula_cmap',
    #'plotBasemapPanel',
    #'pmCI',
    #'regionalMasking',
]

for func in funcs:
    exec(f"from .{func} import *")

mods = [
    "cm",
    "coords",
    "formatting",
    "geoplot",
    "stats",
]

for mod in mods:
    exec(f"from . import {mod}")

__all__ = funcs + mods



#!/usr/bin/env python

import io
import numpy as np
import argparse
import xarray as xr
import warnings
import pkg_resources as pkgr

from omlabs import m6plot
from omlabs.helpers import get_run_name, try_variable_from_list
from omlabs.om4plotting import plot_xydiff, plot_xycompare
from omlabs.om4common import read_data, subset_data
from omlabs.om4common import simple_average, copy_coordinates
from omlabs.om4common import compute_area_regular_grid
from omlabs.om4common import date_range
from omlabs.om4common import image_handler
from omlabs.om4common import open_intake_catalog

from omlabs.om4parser import default_diag_parser

imgbufs = []


def read(dictArgs):
    """read data from model and obs files, process data and return it"""

    dsmodel = xr.open_mfdataset(
        dictArgs["infile"], combine="by_coords", use_cftime=True
    )

    if dictArgs["obsfile"] is not None:
        # priority to user-provided obs file
        dsobs = xr.open_mfdataset(
            dictArgs["obsfile"], combine="by_coords", decode_times=False
        )
    else:
        # use dataset from catalog, either from command line or default
        cat = open_intake_catalog(dictArgs["platform"], "obs")
        dsobs = cat[dictArgs["dataset"]].to_dask()

    # read in model and obs data
    datamodel = read_data(dsmodel, dictArgs["possible_variable_names"])
    dataobs = read_data(dsobs, dictArgs["possible_variable_names"])

    # subset data
    if dictArgs["depth"] is None:
        dictArgs["depth"] = dictArgs["surface_default_depth"]

    if dictArgs["depth"] is not None:
        datamodel = subset_data(datamodel, "assigned_depth", dictArgs["depth"])
        dataobs = subset_data(dataobs, "assigned_depth", dictArgs["depth"])

    # reduce data along depth (not yet implemented)
    if "depth_reduce" in dictArgs:
        if dictArgs["depth_reduce"] == "mean":
            # do mean
            pass
        elif dictArgs["depth_reduce"] == "sum":
            # do sum
            pass

    # reduce data along time, here mandatory
    if ("assigned_time" in datamodel.dims) and (len(datamodel["assigned_time"]) > 1):
        warnings.warn(
            "input dataset has more than one time record, "
            + "performing non-weighted average"
        )
        datamodel = simple_average(datamodel, "assigned_time")
    if ("assigned_time" in dataobs.dims) and len(dataobs["assigned_time"]) > 1:
        warnings.warn(
            "reference dataset has more than one time record, "
            + "performing non-weighted average"
        )
        dataobs = simple_average(dataobs, "assigned_time")

    datamodel = datamodel.squeeze()
    dataobs = dataobs.squeeze()

    # check final data is 2d
    assert len(datamodel.dims) == 2
    assert len(dataobs.dims) == 2

    # check consistency of coordinates
    assert np.allclose(datamodel["assigned_lon"], dataobs["assigned_lon"])
    assert np.allclose(datamodel["assigned_lat"], dataobs["assigned_lat"])

    # homogeneize coords
    dataobs = copy_coordinates(datamodel, dataobs, ["assigned_lon", "assigned_lat"])

    # restrict model to where obs exists
    datamodel = datamodel.where(dataobs)

    # dump values
    model = datamodel.to_masked_array()
    obs = dataobs.to_masked_array()
    x = datamodel["assigned_lon"].values
    y = datamodel["assigned_lat"].values

    # compute area
    if "areacello" in dsmodel.variables:
        area = dsmodel["areacello"].values
    else:
        if model.shape == (180, 360):
            area = compute_area_regular_grid(dsmodel)
        else:
            raise IOError("no cell area provided")

    # date range
    dates = date_range(dsmodel)

    return x, y, area, model, obs, dates


def parse(cliargs=None, template=False):
    """parse the command line arguments"""

    description = "Script for plotting annual-average bias to obs"
    parser = default_diag_parser(
        description=description, template=template, exclude=["basin", "topog"]
    )

    parser.add_argument(
        "--depth",
        type=float,
        default=None,
        required=False,
        help="depth of field compared to obs",
    )

    parser.add_argument(
        "--dataset",
        type=str,
        required=False,
        default="WOA13_annual_TS",
        help="Name of the observational dataset, \
              as provided in intake catalog",
    )

    parser.add_argument(
        "--style",
        type=str,
        required=False,
        default="diff",
        help="output plot style (diff/compare)",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def run(dictArgs):
    """main can be called from either command line and then use parser from run()
    or DORA can build the args and run it directly"""

    # read the data needed for plots
    x, y, area, model, obs, dates = read(dictArgs)
    # make the plots
    figs = plot(x, y, area, model, obs, dates, dictArgs)
    filename = f"{dictArgs['outdir']}/{dictArgs['var']}_{dictArgs['style']}"
    imgbufs = image_handler(figs, dictArgs, filename=filename)

    return imgbufs


def plot(x, y, area, model, obs, dates, dictArgs):
    """meta plotting function"""

    streamdiff = True if dictArgs["style"] == "diff" else False
    streamcompare = True if dictArgs["style"] == "compare" else False
    streamnone = True if dictArgs["style"] is None else False

    # common plot properties
    pngout = dictArgs["outdir"]
    obsds = dictArgs["dataset"]
    var = dictArgs["var"]
    units = dictArgs["units"]
    clim_diff = dictArgs["clim_diff"]
    cmap_diff = dictArgs["cmap_diff"]
    clim_compare = dictArgs["clim_compare"]
    cmap_compare = dictArgs["cmap_compare"]

    if dictArgs["suptitle"] != "":
        suptitle = f"{dictArgs['suptitle']} {dictArgs['label']}"
    else:
        title = get_run_name(dictArgs["infile"])
        suptitle = f"{title} {dictArgs['label']}"

    title_diff = f"{var} bias (w.r.t. {obsds}) {units}   Years {dates[0]} - {dates[1]}"
    title1_compare = f"{var} {units}"
    title2_compare = f"{obsds} {var} {units}"
    png_diff = f"{pngout}/{var}_annual_bias_{obsds}.png"
    png_compare = f"{pngout}/{var}_annual_bias_{obsds}.3_panel.png"

    diff_kwargs = {
        "area": area,
        "suptitle": suptitle,
        "title": title_diff,
        "clim": clim_diff,
        "colormap": cmap_diff,
        "centerlabels": True,
        "extend": "both",
        "save": png_diff,
    }
    compare_kwargs = {
        "area": area,
        "suptitle": suptitle,
        "title1": title1_compare,
        "title2": title2_compare,
        "clim": clim_compare,
        "colormap": cmap_compare,
        "extend": "max",
        "dlim": clim_diff,
        "dcolormap": cmap_diff,
        "dextend": "both",
        "centerdlabels": True,
        "save": png_compare,
    }

    figs = []

    # make diff plot
    if streamdiff or streamnone:
        fig = plot_xydiff(
            x,
            y,
            model,
            obs,
            diff_kwargs,
            interactive=dictArgs["interactive"],
            stream=streamdiff,
        )
        figs.append(fig)

    # make compare plot
    if streamcompare or streamnone:
        fig = plot_xycompare(
            x,
            y,
            model,
            obs,
            compare_kwargs,
            interactive=dictArgs["interactive"],
            stream=streamcompare,
        )
        figs.append(fig)

    return figs


def parse_and_run(cliargs=None):
    cmdLineArgs = parse(cliargs)
    imgbufs = run(cmdLineArgs)
    return imgbufs


if __name__ == "__main__":
    parse_and_run()



#!/usr/bin/env python3

import pkg_resources as pkgr
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import xarray as xr
import xwavelet as xw
import warnings

from omlabs.om4common import image_handler
from omlabs.om4common import open_intake_catalog
from omlabs.om4parser import default_diag_parser

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")

# define NINO regions

nino12 = {
    "lat_range": (-10, 0.0),
    "lon_range": (270.0, 280.0),
    "label": 1.2,
}

nino3 = {
    "lat_range": (-5.0, 5.0),
    "lon_range": (210.0, 270.0),
    "label": 3,
}
nino34 = {
    "lat_range": (-5.0, 5.0),
    "lon_range": (190.0, 240.0),
    "label": 3.4,
}
nino4 = {
    "lat_range": (-5.0, 5.0),
    "lon_range": (160.0, 210.0),
    "label": 4,
}

regions = [nino12, nino3, nino34, nino4]


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = " "

    exclude = ["basin", "hgrid", "topog", "gridspec", "config"]

    parser = default_diag_parser(
        description=description, template=template, exclude=exclude
    )

    parser.add_argument(
        "--varname",
        type=str,
        default="tos",
        help="Variable to analyze. Default is tos.",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs):
    """Read required fields for ENSO analysis

    Parameters
    ----------
    dictArgs : dict
        Dictionary containing argparse options

    Returns
    -------
    (xarray.DataArray, xarray.DataArray)
    """

    # Open model array and static file
    array = xr.open_mfdataset(dictArgs["infile"], use_cftime=True)[dictArgs["varname"]]
    areacello = xr.open_dataset(dictArgs["static"])["areacello"].fillna(0.0)

    # Open pre-calculated ENSO spectra from Obs.
    if dictArgs["obsfile"] is not None:
        ref1 = xr.open_dataset(dictArgs["obsfile"])
        ref1 = ref1["spectrum"]
        ref1.attrs = {**ref1.attrs, "label": "Reference"}
        reference = [ref1]

    else:
        cat = open_intake_catalog(dictArgs["platform"], "obs")

        # open reference datasets
        ref1 = cat["wavelet_NOAA_ERSST_v5_1957_2002"].to_dask()
        ref2 = cat["wavelet_NOAA_ERSST_v5_1880_2019"].to_dask()

        # select spectrum variable
        ref1 = ref1["spectrum"]
        ref2 = ref2["spectrum"]

        # set attributes
        ref1.attrs = {**ref1.attrs, "label": "ERSST v5 1957-2002"}
        ref2.attrs = {**ref2.attrs, "label": "ERSST v5 1880-2019"}

        reference = [ref1, ref2]

    return (array, areacello, reference)


def calculate(array, areacello, timedim="time", reference=None):
    """Main calculation function

    Parameters
    ----------
    array : xarray.DataArray
        Time series data array
    areacello : xarray.DataArray
        Cell area data array
    timedim : str, optional
        Name of time dimension, by default "time"

    Returns
    -------
    [wavelet.Wavelet]
    """

    attrs = array.attrs

    if reference is not None:
        reference = [reference] if not isinstance(reference, list) else reference

    # get a set of variable dimentions excluding "time"
    dims = list(array.dims)
    dims.remove(timedim)
    dims = tuple(dims)

    latdim = "lat"
    londim = "lon"

    array = [
        (
            array.sel({latdim: slice(*x["lat_range"]), londim: slice(*x["lon_range"])})
            .weighted(
                areacello.sel(
                    {latdim: slice(*x["lat_range"]), londim: slice(*x["lon_range"])}
                )
            )
            .mean(dims)
            .assign_attrs({**attrs, "region": f"NINO{x['label']}"}),
            [y.sel(region=x["label"]) for y in reference],
        )
        for x in regions
    ]

    return [xw.Wavelet(x[0], reference=x[1], detrend=False, scaled=True) for x in array]


def plot(results, label):
    """Plotting wrapper for composite plots

    Parameters
    ----------
    results : list of wavelet.Wavelet
        Wavelet class objects
    label : str
        Experiment name

    Returns
    -------
    [matplotlib.figure.Figure]
    """
    fig = [
        x.composite(title=f"ENSO Analysis - {x.dset.timeseries.region}", subtitle=label)
        for x in results
    ]

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence

    Parameters
    ----------
    dictArgs : dict
        Dictionary of parsed options

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # read in data
    array, areacello, reference = read(dictArgs)

    # calculate otsfn
    results = calculate(array, areacello, reference=reference)

    # make the plots
    fig = plot(results, dictArgs["label"])

    filename = [f"{dictArgs['outdir']}/enso_{x['label']}" for x in regions]
    imgbufs = image_handler(fig, dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



"""
A collection of useful functions...
"""
import numpy as np
import tarfile
from scipy.io import netcdf


def section2quadmesh(x, z, q, representation="pcm"):
    """
    Creates the appropriate quadmesh coordinates to plot a scalar q(1:nk,1:ni) at
    horizontal positions x(1:ni+1) and between interfaces at z(nk+1,ni), using
    various representations of the topography.

    Returns X(2*ni+1), Z(nk+1,2*ni+1) and Q(nk,2*ni) to be passed to pcolormesh.

    TBD: Optionally, x can be dimensioned as x(ni) in which case it will be extraplated as if it had
    had dimensions x(ni+1).

    Optional argument:

    representation='pcm' (default) yields a step-wise visualization, appropriate for
             z-coordinate models.
    representation='plm' yields a piecewise-linear visualization more representative
             of general-coordinate (and isopycnal) models.
    representation='linear' is the aesthetically most pleasing but does not
             represent the data conservatively.

    """

    if x.ndim != 1:
        raise Exception("The x argument must be a vector")
    if z.ndim != 2:
        raise Exception("The z argument should be a 2D array")
    if q.ndim != 2:
        raise Exception("The z argument should be a 2D array")
    qnk, qni = q.shape
    znk, zni = z.shape
    xni = x.size
    if zni != qni:
        raise Exception("The last dimension of z and q must be equal in length")
    if znk != qnk + 1:
        raise Exception(
            "The first dimension of z must be 1 longer than that of q. q has %i levels"
            % qnk
        )
    if xni != qni + 1:
        raise Exception("The length of x must 1 longer than the last dimension of q")

    if type(z) == np.ma.core.MaskedArray:
        z[z.mask] = 0
    if type(q) == np.ma.core.MaskedArray:
        qmin = np.amin(q)
        q[q.mask] = qmin

    periodicDomain = (
        abs((x[-1] - x[0]) - 360.0) < 1e-6
    )  # Detect if horizontal axis is a periodic domain

    if representation == "pcm":
        X = np.zeros((2 * qni))
        X[::2] = x[:-1]
        X[1::2] = x[1:]
        Z = np.zeros((qnk + 1, 2 * qni))
        Z[:, ::2] = z
        Z[:, 1::2] = z
        Q = np.zeros((qnk, 2 * qni - 1))
        Q[:, ::2] = q
        Q[:, 1::2] = (q[:, :-1] + q[:, 1:]) / 2.0
    elif representation == "linear":
        X = np.zeros((2 * qni + 1))
        X[::2] = x
        X[1::2] = (x[0:-1] + x[1:]) / 2.0
        Z = np.zeros((qnk + 1, 2 * qni + 1))
        Z[:, 1::2] = z
        Z[:, 2:-1:2] = (z[:, 0:-1] + z[:, 1:]) / 2.0
        Z[:, 0] = z[:, 0]
        Z[:, -1] = z[:, -1]
        Q = np.zeros((qnk, 2 * qni))
        Q[:, ::2] = q
        Q[:, 1::2] = q
    elif representation == "plm":
        X = np.zeros((2 * qni))
        X[::2] = x[:-1]
        X[1::2] = x[1:]
        # PLM reconstruction for Z
        dz = np.roll(z, -1, axis=1) - z  # Right-sided difference
        if not periodicDomain:
            dz[:, -1] = 0  # Non-periodic boundary
        d2 = (
            np.roll(z, -1, axis=1) - np.roll(z, 1, axis=1)
        ) / 2.0  # Centered difference
        d2 = (dz + np.roll(dz, 1, axis=1)) / 2.0  # Centered difference
        s = np.sign(d2)  # Sign of centered slope
        s[dz * np.roll(dz, 1, axis=1) <= 0] = 0  # Flatten extrema
        dz = np.abs(dz)  # Only need magnitude from here on
        S = s * np.minimum(
            np.abs(d2), np.minimum(dz, np.roll(dz, 1, axis=1))
        )  # PLM slope
        Z = np.zeros((qnk + 1, 2 * qni))
        Z[:, ::2] = z - S / 2.0
        Z[:, 1::2] = z + S / 2.0
        Q = np.zeros((qnk, 2 * qni - 1))
        Q[:, ::2] = q
        Q[:, 1::2] = (q[:, :-1] + q[:, 1:]) / 2.0
    else:
        raise Exception("Unknown representation!")

    return X, Z, Q


def get_z(rg, depth, var_name):
    """Returns 3d interface positions from netcdf group rg, based on dimension data for variable var_name"""
    if "e" in rg.variables:  # First try native approach
        if len(rg.variables["e"]) == 3:
            return rg.variables["e"][:]
        elif len(rg.variables["e"]) == 4:
            return rg.variables["e"][0]
    if var_name not in rg.variables:
        raise Exception('Variable "' + var_name + '" not found in netcdf file')
    if len(rg.variables[var_name].shape) < 3:
        raise Exception('Variable "' + var_name + '" must have 3 or more dimensions')
    vdim = rg.variables[var_name].dimensions[-3]
    if vdim not in rg.variables:
        raise Exception(
            'Variable "' + vdim + '" should be a [CF] dimension variable but is missing'
        )
    if "edges" in rg.variables[vdim].ncattrs():
        zvar = getattr(rg.variables[vdim], "edges")
    elif "zw" in rg.variables:
        zvar = "zw"
    else:
        raise Exception(
            'Cannot figure out vertical coordinate from variable "' + var_name + '"'
        )
    if not len(rg.variables[zvar].shape) == 1:
        raise Exception('Variable "' + zvar + '" was expected to be 1d')
    zw = rg.variables[zvar][:]
    Zmod = np.zeros((zw.shape[0], depth.shape[0], depth.shape[1]))
    for k in range(zw.shape[0]):
        Zmod[k] = -np.minimum(depth, abs(zw[k]))
    return Zmod


def rho_Wright97(S, T, P=0):
    """
    Returns the density of seawater for the given salinity, potential temperature
    and pressure.

    Units: salinity in PSU, potential temperature in degrees Celsius and pressure in Pascals.
    """
    a0 = 7.057924e-4
    a1 = 3.480336e-7
    a2 = -1.112733e-7
    b0 = 5.790749e8
    b1 = 3.516535e6
    b2 = -4.002714e4
    b3 = 2.084372e2
    b4 = 5.944068e5
    b5 = -9.643486e3
    c0 = 1.704853e5
    c1 = 7.904722e2
    c2 = -7.984422
    c3 = 5.140652e-2
    c4 = -2.302158e2
    c5 = -3.079464
    al0 = a0 + a1 * T + a2 * S
    p0 = b0 + b4 * S + T * (b1 + T * (b2 + b3 * T) + b5 * S)
    Lambda = c0 + c4 * S + T * (c1 + T * (c2 + c3 * T) + c5 * S)
    return (P + p0) / (Lambda + al0 * (P + p0))


def ice9(i, j, source, xcyclic=True, tripolar=True):
    """
    An iterative (stack based) implementation of "Ice 9".

    The flood fill starts at [j,i] and treats any positive value of "source" as
    passable. Zero and negative values block flooding.

    xcyclic = True allows cyclic behavior in the last index. (default)
    tripolar = True allows a fold across the top-most edge. (default)

    Returns an array of 0's and 1's.
    """
    wetMask = 0 * source
    (nj, ni) = wetMask.shape
    stack = set()
    stack.add((j, i))
    while stack:
        (j, i) = stack.pop()
        if wetMask[j, i] or source[j, i] <= 0:
            continue
        wetMask[j, i] = 1
        if i > 0:
            stack.add((j, i - 1))
        elif xcyclic:
            stack.add((j, ni - 1))
        if i < ni - 1:
            stack.add((j, i + 1))
        elif xcyclic:
            stack.add((j, 0))
        if j > 0:
            stack.add((j - 1, i))
        if j < nj - 1:
            stack.add((j + 1, i))
        elif tripolar:
            stack.add((j, ni - 1 - i))  # Tri-polar fold
    return wetMask


def ice9Wrapper(x, y, depth, xy0):
    ji = nearestJI(x, y, xy0)
    return ice9(ji[1], ji[0], depth)


def maskFromDepth(depth, zCellTop):
    """
    Generates a "wet mask" for a z-coordinate model based on relative location of
    the ocean bottom to the upper interface of the cell.

    depth (2d) is positiveo
    zCellTop (scalar) is a negative position of the upper interface of the cell..
    """
    wet = 0 * depth
    wet[depth > -zCellTop] = 1
    return wet


def MOCpsi(vh, vmsk=None):
    """Sums 'vh' zonally and cumulatively in the vertical to yield an overturning stream function, psi(y,z)."""
    shape = list(vh.shape)
    shape[-3] += 1
    psi = np.zeros(shape[:-1])
    if len(shape) == 3:
        for k in range(shape[-3] - 1, 0, -1):
            if vmsk is None:
                psi[k - 1, :] = psi[k, :] - vh[k - 1].sum(axis=-1)
            else:
                psi[k - 1, :] = psi[k, :] - (vmsk * vh[k - 1]).sum(axis=-1)
    else:
        for n in range(shape[0]):
            for k in range(shape[-3] - 1, 0, -1):
                if vmsk is None:
                    psi[n, k - 1, :] = psi[n, k, :] - vh[n, k - 1].sum(axis=-1)
                else:
                    psi[n, k - 1, :] = psi[n, k, :] - (vmsk * vh[n, k - 1]).sum(axis=-1)
    return psi


def moc_maskedarray(vh, mask=None):
    if mask is not None:
        _mask = np.ma.masked_where(np.not_equal(mask, 1.0), mask)
    else:
        _mask = 1.0
    _vh = vh * _mask
    _vh_btm = np.ma.expand_dims(_vh[:, -1, :, :] * 0.0, axis=1)
    _vh = np.ma.concatenate((_vh, _vh_btm), axis=1)
    _vh = np.ma.sum(_vh, axis=-1) * -1.0
    _vh = _vh[:, ::-1]  # flip z-axis so running sum is from ocean floor to surface
    _vh = np.ma.cumsum(_vh, axis=1)
    _vh = _vh[:, ::-1]  # flip z-axis back to original order
    return _vh


def nearestJI(x, y, xy0):
    """
    Find (j,i) of cell with center nearest to (x0,y0).
    """
    x0, y0 = xy0
    return np.unravel_index(((x - x0) ** 2 + (y - y0) ** 2).argmin(), x.shape)


def readNCFromTar(tar, file, var):
    TF = tarfile.open(tar, "r")
    member = [m for m in TF.getmembers() if file in m.name][0]
    nc = netcdf.netcdf_file(TF.extractfile(member), "r")
    return nc.variables[var]


def southOf(x, y, xy0, xy1):
    """
    Returns 1 for point south/east of the line that passes through xy0-xy1, 0 otherwise.
    """
    x0 = xy0[0]
    y0 = xy0[1]
    x1 = xy1[0]
    y1 = xy1[1]
    dx = x1 - x0
    dy = y1 - y0
    Y = (x - x0) * dy - (y - y0) * dx
    Y[Y >= 0] = 1
    Y[Y <= 0] = 0
    return Y


def genBasinMasks(x, y, depth, verbose=False):
    if verbose:
        print("Generating global wet mask ...")
    wet = ice9Wrapper(
        x, y, depth, (0, -35)
    )  # All ocean points seeded from South Atlantic
    if verbose:
        print("done.")

    code = 0 * wet

    if verbose:
        print("Finding Cape of Good Hope ...")
    tmp = 1 - wet
    tmp[x < -30] = 0
    tmp = ice9Wrapper(x, y, tmp, (20, -30.0))
    yCGH = (tmp * y).min()
    if verbose:
        print("done.", yCGH)

    if verbose:
        print("Finding Melbourne ...")
    tmp = 1 - wet
    tmp[x > -180] = 0
    tmp = ice9Wrapper(x, y, tmp, (-220, -25.0))
    yMel = (tmp * y).min()
    if verbose:
        print("done.", yMel)

    if verbose:
        print("Processing Persian Gulf ...")
    tmp = wet * (1 - southOf(x, y, (55.0, 23.0), (56.5, 27.0)))
    tmp = ice9Wrapper(x, y, tmp, (53.0, 25.0))
    code[tmp > 0] = 11
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Red Sea ...")
    tmp = wet * (1 - southOf(x, y, (40.0, 11.0), (45.0, 13.0)))
    tmp = ice9Wrapper(x, y, tmp, (40.0, 18.0))
    code[tmp > 0] = 10
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Black Sea ...")
    tmp = wet * (1 - southOf(x, y, (26.0, 42.0), (32.0, 40.0)))
    tmp = ice9Wrapper(x, y, tmp, (32.0, 43.0))
    code[tmp > 0] = 7
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Mediterranean ...")
    tmp = wet * (southOf(x, y, (-5.7, 35.5), (-5.7, 36.5)))
    tmp = ice9Wrapper(x, y, tmp, (4.0, 38.0))
    code[tmp > 0] = 6
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Baltic ...")
    tmp = wet * (southOf(x, y, (8.6, 56.0), (8.6, 60.0)))
    tmp = ice9Wrapper(x, y, tmp, (10.0, 58.0))
    code[tmp > 0] = 9
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Hudson Bay ...")
    tmp = wet * (
        (
            1
            - (1 - southOf(x, y, (-95.0, 66.0), (-83.5, 67.5)))
            * (1 - southOf(x, y, (-83.5, 67.5), (-84.0, 71.0)))
        )
        * (1 - southOf(x, y, (-70.0, 58.0), (-70.0, 65.0)))
    )
    tmp = ice9Wrapper(x, y, tmp, (-85.0, 60.0))
    code[tmp > 0] = 8
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Arctic ...")
    tmp = wet * (
        (1 - southOf(x, y, (-171.0, 66.0), (-166.0, 65.5)))
        * (1 - southOf(x, y, (-64.0, 66.4), (-50.0, 68.5)))  # Lab Sea
        + southOf(x, y, (-50.0, 0.0), (-50.0, 90.0))
        * (1 - southOf(x, y, (0.0, 65.5), (360.0, 65.5)))  # Denmark Strait
        + southOf(x, y, (-18.0, 0.0), (-18.0, 65.0))
        * (1 - southOf(x, y, (0.0, 64.9), (360.0, 64.9)))  # Iceland-Sweden
        + southOf(x, y, (20.0, 0.0), (20.0, 90.0))  # Barents Sea
        + (1 - southOf(x, y, (-280.0, 55.0), (-200.0, 65.0)))
    )
    tmp = ice9Wrapper(x, y, tmp, (0.0, 85.0))
    code[tmp > 0] = 4
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Pacific ...")
    tmp = wet * (
        (1 - southOf(x, y, (0.0, yMel), (360.0, yMel)))
        - southOf(x, y, (-257, 1), (-257, 0)) * southOf(x, y, (0, 3), (1, 3))
        - southOf(x, y, (-254.25, 1), (-254.25, 0)) * southOf(x, y, (0, -5), (1, -5))
        - southOf(x, y, (-243.7, 1), (-243.7, 0)) * southOf(x, y, (0, -8.4), (1, -8.4))
        - southOf(x, y, (-234.5, 1), (-234.5, 0)) * southOf(x, y, (0, -8.9), (1, -8.9))
    )
    tmp = ice9Wrapper(x, y, tmp, (-150.0, 0.0))
    code[tmp > 0] = 3
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Atlantic ...")
    tmp = wet * (1 - southOf(x, y, (0.0, yCGH), (360.0, yCGH)))
    tmp = ice9Wrapper(x, y, tmp, (-20.0, 0.0))
    code[tmp > 0] = 2
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Indian ...")
    tmp = wet * (1 - southOf(x, y, (0.0, yCGH), (360.0, yCGH)))
    tmp = ice9Wrapper(x, y, tmp, (55.0, 0.0))
    code[tmp > 0] = 5
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Processing Southern Ocean ...")
    tmp = ice9Wrapper(x, y, wet, (0.0, -55.0))
    code[tmp > 0] = 1
    wet = wet - tmp  # Removed named points

    if verbose:
        print("Remapping Persian Gulf points to the Indian Ocean for OMIP/CMIP6 ...")
    code[code == 11] = 5

    code[wet > 0] = -9
    (j, i) = np.unravel_index(wet.argmax(), x.shape)
    if j:
        if verbose:
            print("There are leftover points unassigned to a basin code")
        while j:
            print(x[j, i], y[j, i], [j, i])
            wet[j, i] = 0
            (j, i) = np.unravel_index(wet.argmax(), x.shape)
    else:
        if verbose:
            print("All points assigned a basin code")

    if verbose:
        print(
            """
Basin codes:
-----------------------------------------------------------
  (1) Southern Ocean      (6) Mediterranean Sea
  (2) Atlantic Ocean      (7) Black Sea
  (3) Pacific Ocean       (8) Hudson Bay
  (4) Arctic Ocean        (9) Baltic Sea
  (5) Indian Ocean       (10) Red Sea

    """
        )

    return code


# Tests
if __name__ == "__main__":

    import matplotlib.pyplot as plt
    import numpy.matlib

    # Test data
    x = np.arange(5)
    z = (
        np.array(
            [
                [0, 0.2, 0.3, -0.1],
                [1, 1.5, 0.7, 0.4],
                [2, 2, 1.5, 2],
                [3, 2.3, 1.5, 2.1],
            ]
        )
        * -1
    )
    q = np.matlib.rand(3, 4)
    print("x=", x)
    print("z=", z)
    print("q=", q)

    X, Z, Q = section2quadmesh(x, z, q)
    print("X=", X)
    print("Z=", Z)
    print("Q=", Q)
    plt.subplot(3, 1, 1)
    plt.pcolormesh(X, Z, Q)

    X, Z, Q = section2quadmesh(x, z, q, representation="linear")
    print("X=", X)
    print("Z=", Z)
    print("Q=", Q)
    plt.subplot(3, 1, 2)
    plt.pcolormesh(X, Z, Q)

    X, Z, Q = section2quadmesh(x, z, q, representation="plm")
    print("X=", X)
    print("Z=", Z)
    print("Q=", Q)
    plt.subplot(3, 1, 3)
    plt.pcolormesh(X, Z, Q)

    plt.show()



import matplotlib.pyplot as plt
from matplotlib.colors import BoundaryNorm, ListedColormap, LogNorm
from matplotlib.ticker import MaxNLocator


def chooseColorLevels(
    sMin,
    sMax,
    colorMapName,
    clim=None,
    nbins=None,
    steps=[1, 2, 2.5, 5, 10],
    extend=None,
    logscale=False,
    autocenter=False,
):
    """
    If nbins is a positive integer, choose sensible color levels with nbins colors.
    If clim is a 2-element tuple, create color levels within the clim range
    or if clim is a vector, use clim as contour levels.
    If clim provides more than 2 color interfaces, nbins must be absent.
    If clim is absent, the sMin,sMax are used as the color range bounds.
    If autocenter is True and clim is None then the automatic color levels are centered.

    Returns cmap, norm and extend.
    """
    if nbins is None and clim is None:
        raise Exception("At least one of clim or nbins is required.")
    if clim is not None:
        if len(clim) < 2:
            raise Exception("clim must be at least 2 values long.")
        if nbins is None and len(clim) == 2:
            raise Exception(
                "nbins must be provided when clims specifies a color range."
            )
        if nbins is not None and len(clim) > 2:
            raise Exception(
                "nbins cannot be provided when clims specifies color levels."
            )
    if clim is None:
        if autocenter:
            levels = MaxNLocator(nbins=nbins, steps=steps).tick_values(
                min(sMin, -sMax), max(sMax, -sMin)
            )
        else:
            levels = MaxNLocator(nbins=nbins, steps=steps).tick_values(sMin, sMax)
    elif len(clim) == 2:
        levels = MaxNLocator(nbins=nbins, steps=steps).tick_values(clim[0], clim[1])
    else:
        levels = clim

    nColors = len(levels) - 1
    if extend is None:
        if sMin < levels[0] and sMax > levels[-1]:
            extend = "both"  # ; eColors=[1,1]
        elif sMin < levels[0] and sMax <= levels[-1]:
            extend = "min"  # ; eColors=[1,0]
        elif sMin >= levels[0] and sMax > levels[-1]:
            extend = "max"  # ; eColors=[0,1]
        else:
            extend = "neither"  # ; eColors=[0,0]
    eColors = [0, 0]
    if extend in ["both", "min"]:
        eColors[0] = 1
    if extend in ["both", "max"]:
        eColors[1] = 1

    cmap = plt.get_cmap(colorMapName)  # ,lut=nColors+eColors[0]+eColors[1])
    # cmap0 = cmap(0.)
    # cmap1 = cmap(1.)
    # cmap = ListedColormap(cmap(range(eColors[0],nColors+1-eColors[1]+eColors[0])))#, N=nColors)
    # if eColors[0]>0: cmap.set_under(cmap0)
    # if eColors[1]>0: cmap.set_over(cmap1)
    if logscale:
        norm = LogNorm(vmin=levels[0], vmax=levels[-1])
    else:
        norm = BoundaryNorm(levels, ncolors=cmap.N)
    return cmap, norm, extend



import matplotlib.pyplot as plt
import numpy as np


def addStatusBar(xCoord, yCoord, zData):
    """
    Reformats status bar message
    """

    class hiddenStore:
        def __init__(self, axis):
            self.axis = axis
            self.xMin, self.xMax = axis.get_xlim()
            self.yMin, self.yMax = axis.get_ylim()

    save = hiddenStore(plt.gca())

    def statusMessage(x, y):
        # THIS NEEDS TESTING FOR ACCURACY, ESPECIALLY IN YZ PLOTS -AJA
        if len(xCoord.shape) == 1 and len(yCoord.shape) == 1:
            # -2 needed because of coords are for vertices and need to be averaged to centers
            i = min(
                range(len(xCoord) - 2),
                key=lambda l: abs((xCoord[l] + xCoord[l + 1]) / 2.0 - x),
            )
            j = min(
                range(len(yCoord) - 2),
                key=lambda l: abs((yCoord[l] + yCoord[l + 1]) / 2.0 - y),
            )
        elif len(xCoord.shape) == 1 and len(yCoord.shape) == 2:
            i = min(
                range(len(xCoord) - 2),
                key=lambda l: abs((xCoord[l] + xCoord[l + 1]) / 2.0 - x),
            )
            j = min(
                range(len(yCoord[:, i]) - 1),
                key=lambda l: abs((yCoord[l, i] + yCoord[l + 1, i]) / 2.0 - y),
            )
        elif len(xCoord.shape) == 2 and len(yCoord.shape) == 2:
            idx = np.abs(
                np.fabs(
                    xCoord[0:-1, 0:-1]
                    + xCoord[1:, 1:]
                    + xCoord[0:-1, 1:]
                    + xCoord[1:, 0:-1]
                    - 4 * x
                )
                + np.fabs(
                    yCoord[0:-1, 0:-1]
                    + yCoord[1:, 1:]
                    + yCoord[0:-1, 1:]
                    + yCoord[1:, 0:-1]
                    - 4 * y
                )
            ).argmin()
            j, i = np.unravel_index(idx, zData.shape)
        else:
            raise Exception("Combindation of coordinates shapes is VERY UNUSUAL!")
        if not i is None:
            val = zData[j, i]
            if val is np.ma.masked:
                return "x,y=%.3f,%.3f  f(%i,%i)=NaN" % (x, y, i + 1, j + 1)
            else:
                return "x,y=%.3f,%.3f  f(%i,%i)=%g" % (x, y, i + 1, j + 1, val)
        else:
            return "x,y=%.3f,%.3f" % (x, y)

    plt.gca().format_coord = statusMessage



#!/usr/bin/env python3

import argparse
import pkg_resources as pkgr
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import palettable
import xarray as xr
import warnings

from omlabs.om4common import horizontal_grid
from omlabs.om4common import image_handler
from omlabs.om4common import date_range
from omlabs.om4common import open_intake_catalog
from omlabs.om4parser import default_diag_parser

warnings.filterwarnings("ignore", message=".*csr_matrix.*")
warnings.filterwarnings("ignore", message=".*dates out of range.*")


def generate_basin_masks(basin_code, basin=None):
    """Function to generate pre-defined basin masks"""
    mask = basin_code * 0
    if basin == "atlantic_arctic":
        mask[
            (basin_code == 2)
            | (basin_code == 4)
            | (basin_code == 6)
            | (basin_code == 7)
            | (basin_code == 8)
        ] = 1.0
    elif basin == "indo_pacific":
        mask[(basin_code == 3) | (basin_code == 5)] = 1.0
    elif isinstance(basin, int):
        mask[(basin_code == basin)] = 1.0
    else:
        mask[(basin_code >= 1)] = 1.0
    return mask


def calculate(advective, diffusive=None, vmask=None, rho0=1.035e3, Cp=3989.0):
    """Converts vertically integrated temperature advection into heat transport"""

    if diffusive is not None:
        HT = advective + diffusive
    else:
        HT = advective
    if len(HT.time) > 1:
        HT = HT.mean(dim="time")
        warnings.warn("Performing non-weighted time average.")

    if advective.units == "Celsius meter3 second-1":
        HT = HT * (rho0 * Cp)
        HT = HT * 1.0e-15  # convert to PW
    elif (
        advective.units == "W m-2"
    ):  # bug in MOM6 units (issue #934), keep for retrocompatibility
        HT = HT * 1.0e-15
    elif advective.units == "W":
        HT = HT * 1.0e-15
    else:
        print("Unknown units")

    HT = HT.to_masked_array()

    if vmask is not None:
        HT = HT * vmask

    HT = HT.sum(axis=-1)
    HT = HT.squeeze()  # sum in x-direction

    return HT


def parse(cliargs=None, template=False):
    """
    Function to capture the user-specified command line options
    """
    description = """ """

    parser = default_diag_parser(
        description=description, template=template, exclude=["obsfile", "topog"]
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs, adv_varname="T_ady_2d", dif_varname="T_diffy_2d"):
    """Read in heat transport data"""

    infile = dictArgs["infile"]
    ds = xr.open_mfdataset(infile, combine="by_coords")

    # advective component of transport
    advective = ds[adv_varname]

    # diffusive component of transport
    if dif_varname in ds.variables:
        diffusive = ds[dif_varname]
    else:
        diffusive = None

    outputgrid = "nonsymetric"
    dsV = horizontal_grid(dictArgs, point_type="v", outputgrid=outputgrid)
    if advective.shape[-2] == (dsV.geolat.shape[0] + 1):
        print("Symmetric grid detected.<br>")
        outputgrid = "symetric"
        dsV = horizontal_grid(dictArgs, point_type="v", outputgrid=outputgrid)

    geolon_v = dsV.geolon.values
    geolat_v = dsV.geolat.values
    yq = dsV.nominal_y.values
    basin_code = dsV.basin.values

    # basin masks
    atlantic_arctic_mask = generate_basin_masks(basin_code, basin="atlantic_arctic")
    indo_pacific_mask = generate_basin_masks(basin_code, basin="indo_pacific")

    # date range
    dates = date_range(ds)

    return (
        geolon_v,
        geolat_v,
        yq,
        basin_code,
        atlantic_arctic_mask,
        indo_pacific_mask,
        advective,
        diffusive,
        dates,
    )


class GWObs:
    class _gw:
        def __init__(self, lat, trans, err):
            self.lat = lat
            self.trans = trans
            self.err = err
            self.minerr = trans - err
            self.maxerr = trans + err

        def annotate(self, ax):
            for n in range(0, len(self.minerr)):
                if n == 0:
                    ax.plot(
                        [self.lat[n], self.lat[n]],
                        [self.minerr[n], self.maxerr[n]],
                        "c",
                        linewidth=2.0,
                        label="G&W",
                    )
                else:
                    ax.plot(
                        [self.lat[n], self.lat[n]],
                        [self.minerr[n], self.maxerr[n]],
                        "c",
                        linewidth=2.0,
                    )
                ax.scatter(self.lat, self.trans, marker="s", facecolor="cyan")

    def __init__(self):
        self.gbl = self._gw(
            np.array([-30.0, -19.0, 24.0, 47.0]),
            np.array([-0.6, -0.8, 1.8, 0.6]),
            np.array([0.3, 0.6, 0.3, 0.1]),
        )
        self.atl = self._gw(
            np.array([-45.0, -30.0, -19.0, -11.0, -4.5, 7.5, 24.0, 47.0]),
            np.array([0.66, 0.35, 0.77, 0.9, 1.0, 1.26, 1.27, 0.6]),
            np.array([0.12, 0.15, 0.2, 0.4, 0.55, 0.31, 0.15, 0.09]),
        )
        self.indpac = self._gw(
            np.array([-30.0, -18.0, 24.0, 47.0]),
            np.array([-0.9, -1.6, 0.52, 0.0]),
            np.array(
                [
                    0.3,
                    0.6,
                    0.2,
                    0.05,
                ]
            ),
        )


def plot(dictArgs, yq, trans_global, trans_atlantic, trans_pacific, dates=None):

    # Load observations for plotting
    GW = GWObs()

    cat = open_intake_catalog(dictArgs["platform"], "obs")
    fObs = cat["Trenberth_and_Caron"].to_dask()

    yobs = fObs.ylat.to_masked_array()
    NCEP_Global = fObs.OTn.to_masked_array()
    NCEP_Atlantic = fObs.ATLn.to_masked_array()
    NCEP_IndoPac = fObs.INDPACn.to_masked_array()
    ECMWF_Global = fObs.OTe.to_masked_array()
    ECMWF_Atlantic = fObs.ATLe.to_masked_array()
    ECMWF_IndoPac = fObs.INDPACe.to_masked_array()

    fig = plt.figure(figsize=(6, 10))

    # Global Heat Transport
    ax1 = plt.subplot(3, 1, 1)
    plt.plot(yq, yq * 0.0, "k", linewidth=0.5)
    plt.plot(yq, trans_global, "r", linewidth=1.5, label="Model")
    GW.gbl.annotate(ax1)
    plt.plot(yobs, NCEP_Global, "k--", linewidth=0.5, label="NCEP")
    plt.plot(yobs, ECMWF_Global, "k.", linewidth=0.5, label="ECMWF")
    plt.ylim(-2.5, 3.0)
    plt.grid(True)
    plt.legend(loc=2, fontsize=10)
    ax1.text(
        0.01,
        1.02,
        "a. Global Poleward Heat Transport",
        ha="left",
        transform=ax1.transAxes,
    )

    if dates is not None:
        assert isinstance(dates, tuple), "Year range should be provided as a tuple."
        datestring = f"Years {dates[0]} - {dates[1]}"
        ax1.text(
            0.98, 1.02, datestring, ha="right", fontsize=10, transform=ax1.transAxes
        )

    # if diffusive is None: annotatePlot('Warning: Diffusive component of transport is missing.')

    # Atlantic Heat Transport
    ax2 = plt.subplot(3, 1, 2)
    plt.plot(yq, yq * 0.0, "k", linewidth=0.5)
    trans_atlantic[yq <= -33] = np.nan
    plt.plot(yq, trans_atlantic, "r", linewidth=1.5, label="Model")
    GW.atl.annotate(ax2)
    plt.plot(yobs, NCEP_Atlantic, "k--", linewidth=0.5, label="NCEP")
    plt.plot(yobs, ECMWF_Atlantic, "k.", linewidth=0.5, label="ECMWF")
    plt.ylim(-0.5, 2.0)
    plt.grid(True)
    ax2.text(
        0.01,
        1.02,
        "b. Atlantic Poleward Heat Transport",
        ha="left",
        transform=ax2.transAxes,
    )

    # Indo-pacific Heat Transport
    ax3 = plt.subplot(3, 1, 3)
    plt.plot(yq, yq * 0.0, "k", linewidth=0.5)
    trans_pacific[yq <= -33] = np.nan
    plt.plot(yq, trans_pacific, "r", linewidth=1.5, label="Model")
    GW.indpac.annotate(ax3)
    plt.plot(yobs, NCEP_IndoPac, "k--", linewidth=0.5, label="NCEP")
    plt.plot(yobs, ECMWF_IndoPac, "k.", linewidth=0.5, label="ECMWF")
    plt.ylim(-2.5, 1.5)
    plt.grid(True)
    plt.xlabel(r"Latitude [$\degree$N]")
    ax3.text(
        0.01,
        1.02,
        "c. Indo-Pacific Poleward Heat Transport",
        ha="left",
        transform=ax3.transAxes,
    )

    plt.subplots_adjust(hspace=0.3)

    # Annotations
    fig.text(
        0.05,
        0.05,
        r"Trenberth, K. E. and J. M. Caron, 2001: Estimates of Meridional Atmosphere and Ocean Heat Transports. J.Climate, 14, 3433-3443.",
        fontsize=6,
    )
    fig.text(
        0.05,
        0.04,
        r"Ganachaud, A. and C. Wunsch, 2000: Improved estimates of global ocean circulation, heat transport and mixing from hydrographic data.",
        fontsize=6,
    )
    fig.text(0.05, 0.03, r"Nature, 408, 453-457", fontsize=6)

    if dictArgs["label"] is not None:
        plt.suptitle(dictArgs["label"])

    # HTplot = heatTrans(advective, diffusive, vmask=m*numpy.roll(m,-1,axis=-2))

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence"""

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # --- the main show ---
    (
        x,
        y,
        yq,
        basin_code,
        atlantic_arctic_mask,
        indo_pacific_mask,
        advective,
        diffusive,
        dates,
    ) = read(dictArgs)

    trans_global = calculate(advective, diffusive)
    trans_atlantic = calculate(advective, diffusive, vmask=atlantic_arctic_mask)
    trans_pacific = calculate(advective, diffusive, vmask=indo_pacific_mask)

    fig = plot(dictArgs, yq, trans_global, trans_atlantic, trans_pacific, dates)

    filename = f"{dictArgs['outdir']}/heat_transport"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



#!/usr/bin/env python3

import argparse
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import xarray as xr

from omlabs.om4common import image_handler
from omlabs.om4parser import default_diag_parser


def parse(cliargs=None, template=False):
    """
    Function to capture the user-specified command line options
    """
    description = """Generic routine for plotting section transports"""

    exclude = [
        "basin",
        "config",
        "gridspec",
        "hgrid",
        "obsfile",
        "platform",
        "static",
        "suptitle",
        "topog",
    ]

    parser = default_diag_parser(
        description=description, template=template, exclude=exclude
    )

    parser.add_argument(
        "--passage_label",
        type=str,
        default="",
        help="Text to use for the name of the passage being plotted",
    )

    parser.add_argument(
        "--obsrange",
        type=str,
        default=None,
        help="Comma-separated tuple of min and max observed ranges (e.g. -20,20)",
    )

    parser.add_argument(
        "--outfile",
        type=str,
        default="section",
        help="Name of output file name with path (--outdir) and extension (--format)",
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(infile, varlist=["umo", "vmo"]):
    """Reads input files from a working directory"""

    ds = xr.open_mfdataset(infile, use_cftime=True, combine="by_coords")
    assert any(
        field in ds.variables for field in varlist
    ), f"Input dataset must include one of {varlist}"
    return ds


def calculate(
    dset_transport,
    components=["umo", "vmo"],
    zlim=None,
    conversion=1.0e-9,
    monthavg=True,
):
    """Function to calculate the transport normal to defined passage"""

    # cross-check variable list with input dataset
    complist = [
        varname for varname in components if varname in dset_transport.variables
    ]

    # transport will eventually contain the total transport, from both
    # u and v components if provided. It is initialized here as a
    # list of xarray.DataArrays()
    transport = []

    # loop over transport components
    for comp in complist:

        # obtain a list of dimensions
        component = dset_transport[comp]
        dims = list(component.dims)

        # detect x and y dimensions for this DataArray
        xdim = [dim for dim in dims if dim.startswith("x")]
        ydim = [dim for dim in dims if dim.startswith("y")]

        if len(xdim) != 1:
            raise RuntimeError(f"Unable to find x dimension in {comp}")
        if len(ydim) != 1:
            raise RuntimeError(f"Unable to find y dimension in {comp}")

        xdim = xdim[0]
        ydim = ydim[0]

        # detect and sum the transport along the dimension normal to the passage definition
        avedim = ydim if len(dset_transport[ydim]) > len(dset_transport[xdim]) else xdim
        component = component.sum(dim=avedim)

        # ensure we have a 3-d data cube at this step.
        assert len(component.dims) == 3, f"{comp} may be missing a time coordinate"

        # The data cube should also contain a singleton dimension at this step
        component = component.squeeze()
        assert len(component.dims) == 2, f"{comp} did not reduce to (time,depth)"

        # Append this array to the total transport
        transport.append(component)

    # concatentate components along a new axis and take the sum
    transport = xr.concat(transport, dim="component")
    transport = transport.sum(dim="component")
    assert (
        len(transport.dims) == 2
    ), f"Total transport has wrong dimensions, should be (time,depth)"

    # restrict vertical range if asked
    zcoord = transport.dims[1]
    if zlim is not None:
        assert isinstance(zlim, tuple), "zlim must be a tuple of depths"
        zlim = {zcoord: slice(*zlim)}
        transport = transport.sel(**zlim)

    # sum in the verical
    transport = transport.sum(dim=zcoord)

    # check that we have a timeseries
    assert len(transport.dims) == 1, "Dataset did not yield a timeseries"

    # apply conversion factor
    transport = transport * conversion

    # create montly average if asked
    transport = transport.resample(time="1M").mean() if monthavg else transport

    # rename the output field
    transport = transport.rename("summed transport")

    # load array
    transport.load()

    return transport


def plot(transport, ylim=None, label=None, passage_label=None, obsrange=None):
    fig = plt.figure(figsize=(8.5, 4))
    ax = plt.subplot(1, 1, 1)

    # plot obs range
    if obsrange is not None:
        minval = min(obsrange) * np.ones((len(transport.values)))
        maxval = max(obsrange) * np.ones((len(transport.values)))
        plt.fill_between(
            transport.time.values,
            minval,
            maxval,
            color="black",
            alpha=0.2,
            edgecolor=None,
        )

    ax.plot(transport.time, transport.values, color="k")

    ax.text(0.02, 1.02, passage_label, ha="left", fontsize=16, transform=ax.transAxes)
    ax.text(0.98, 1.02, label, ha="right", fontsize=12, transform=ax.transAxes)

    ax.set_ylim(ylim)

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence"""

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")
    else:
        plt.switch_backend("TkAgg")

    # --- the main show ---

    dset_transport = read(dictArgs["infile"])
    transport = calculate(dset_transport)

    dictArgs["obsrange"] = (
        tuple([float(x) for x in dictArgs["obsrange"].split(",")])
        if dictArgs["obsrange"] is not None
        else None
    )

    fig = plot(
        transport,
        label=dictArgs["label"],
        passage_label=dictArgs["passage_label"],
        obsrange=dictArgs["obsrange"],
    )

    # ---------------------

    # construct output filename based on "passage_label" argument, if present
    filename = dictArgs["outfile"]
    filename = filename.replace(" ", "_")
    filename = f"{dictArgs['outdir']}/{filename}"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



import matplotlib.pyplot as plt


def addInteractiveCallbacks():
    """
    Adds interactive features to a plot on screen.
    Key 'q' to close window.
    Zoom button to center.
    Zoom wheel to zoom in and out.
    """

    def keyPress(event):
        if event.key == "Q":
            exit(0)  # Exit python
        elif event.key == "q":
            plt.close()  # Close just the active figure

    class hiddenStore:
        def __init__(self, axis):
            self.axis = axis
            self.xMin, self.xMax = axis.get_xlim()
            self.yMin, self.yMax = axis.get_ylim()

    save = hiddenStore(plt.gca())

    def zoom(event):  # Scroll wheel up/down
        if event.button == "up":
            scaleFactor = 1 / 1.5  # deal with zoom in
        elif event.button == "down":
            scaleFactor = 1.5  # deal with zoom out
        elif event.button == 2:
            scaleFactor = 1.0
        else:
            return
        axis = event.inaxes
        axmin, axmax = axis.get_xlim()
        aymin, aymax = axis.get_ylim()
        (axmin, axmax), (aymin, aymax) = newLims(
            (axmin, axmax),
            (aymin, aymax),
            (event.xdata, event.ydata),
            (save.xMin, save.xMax),
            (save.yMin, save.yMax),
            scaleFactor,
        )
        if axmin is None:
            return
        for axis in plt.gcf().get_axes():
            if axis.get_navigate():
                axis.set_xlim(axmin, axmax)
                axis.set_ylim(aymin, aymax)
        plt.draw()  # force re-draw

    def zoom2(event):
        zoom(event)

    plt.gcf().canvas.mpl_connect("key_press_event", keyPress)
    plt.gcf().canvas.mpl_connect("scroll_event", zoom)
    plt.gcf().canvas.mpl_connect("button_press_event", zoom2)



import io

try:
    from omlabs import m6plot
except ImportError:
    import m6plot


def plot_xydiff(x, y, slice1, slice2, diff_kwargs, interactive=False, stream=False):
    """make difference plot"""
    if stream:
        img = io.BytesIO()
        diff_kwargs["save"] = img

    fig = m6plot.xyplot(slice1 - slice2, x, y, interactive=interactive, **diff_kwargs)

    return fig


def plot_xycompare(
    x, y, slice1, slice2, compare_kwargs, interactive=False, stream=False
):
    """make 3 panel compare plot"""
    if stream:
        img = io.BytesIO()
        compare_kwargs["save"] = img

    fig = m6plot.xycompare(
        slice1, slice2, x, y, interactive=interactive, **compare_kwargs
    )

    return fig


def plot_yzdiff(y, z, slice1, slice2, diff_kwargs, interactive=False, stream=False):
    """make difference plot"""
    if stream:
        img = io.BytesIO()
        diff_kwargs["save"] = img

    fig = m6plot.yzplot(slice1 - slice2, y, z, interactive=interactive, **diff_kwargs)

    return fig


def plot_yzcompare(
    y, z, slice1, slice2, compare_kwargs, interactive=False, stream=False
):
    """make 3 panel compare plot"""
    if stream:
        img = io.BytesIO()
        compare_kwargs["save"] = img

    fig = m6plot.yzcompare(
        slice1, slice2, y, z, interactive=interactive, **compare_kwargs
    )

    return fig



def cmoceanRegisterColormaps():
    import matplotlib

    matplotlib.cm.register_cmap(name="algae", cmap=cmocean.cm.algae)
    matplotlib.cm.register_cmap(name="thermal", cmap=cmocean.cm.thermal)
    matplotlib.cm.register_cmap(name="haline", cmap=cmocean.cm.haline)
    matplotlib.cm.register_cmap(name="solar", cmap=cmocean.cm.solar)
    matplotlib.cm.register_cmap(name="ice", cmap=cmocean.cm.ice)
    matplotlib.cm.register_cmap(name="gray", cmap=cmocean.cm.gray)
    matplotlib.cm.register_cmap(name="oxy", cmap=cmocean.cm.oxy)
    matplotlib.cm.register_cmap(name="deep", cmap=cmocean.cm.deep)
    matplotlib.cm.register_cmap(name="dense", cmap=cmocean.cm.dense)
    matplotlib.cm.register_cmap(name="algae", cmap=cmocean.cm.algae)
    matplotlib.cm.register_cmap(name="matter", cmap=cmocean.cm.matter)
    matplotlib.cm.register_cmap(name="turbid", cmap=cmocean.cm.turbid)
    matplotlib.cm.register_cmap(name="speed", cmap=cmocean.cm.speed)
    matplotlib.cm.register_cmap(name="amp", cmap=cmocean.cm.amp)
    matplotlib.cm.register_cmap(name="tempo", cmap=cmocean.cm.tempo)
    matplotlib.cm.register_cmap(name="phase", cmap=cmocean.cm.phase)
    matplotlib.cm.register_cmap(name="balance", cmap=cmocean.cm.balance)
    matplotlib.cm.register_cmap(name="delta", cmap=cmocean.cm.delta)
    matplotlib.cm.register_cmap(name="curl", cmap=cmocean.cm.curl)



from .thetao_yz_annual_bias_1x1deg import parse_and_run, parse, run

__description__ = "potential tempertaure depth-latitude annual bias plot"
__ppstreams__ = ["ocean_annual_z_d2_1x1deg/av", "ocean_annual_z_1x1deg/av"]



#!/usr/bin/env python3

import pkg_resources as pkgr
import intake
import io
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from omlabs import m6plot
import palettable
import xarray as xr
import xoverturning
import warnings

# these are always needed
from omlabs.om4parser import default_diag_parser
from omlabs.om4common import image_handler

# these are available to you to use
from omlabs.om4common import date_range
from omlabs.om4common import generate_basin_masks
from omlabs.om4common import horizontal_grid
from omlabs.om4common import is_symmetric
from omlabs.om4common import read_topography


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = " "

    parser = default_diag_parser(description=description, template=template)

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def read(dictArgs):
    """Read required fields

    Parameters
    ----------
    dictArgs : dict
        Dictionary containing argparse options

    Returns
    -------
    xarray.DataSet
    """

    dset = xr.open_mfdataset(dictArgs["infile"], use_cftime=True)
    dset_static = xr.open_dataset(dictArgs["static"])

    return (dset, dset_static)


def calculate(dset, dset_static):
    """Main calculation function

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset
    dset_static : xarray.Dataset
        Input static dataset

    Returns
    -------
    xarray.DataArray
    """

    return xr.DataArray()


def plot(arr, label=None):
    """Plotting wrapper

    Parameters
    ----------
    arr : xarray.DataArray
        Input data array

    Returns
    -------
    matplotlib.figure.Figure
        Figure handle
    """

    fig = plt.figure()
    ax = plt.subplot(1, 1, 1)

    ax.plot(arr.values)

    return fig


def run(dictArgs):
    """Function to call read, calc, and plot in sequence

    Parameters
    ----------
    dictArgs : dict
        Dictionary of parsed options

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """

    # set visual backend
    if dictArgs["interactive"] is False:
        plt.switch_backend("Agg")

    # read in data
    dset, dset_static = read(dictArgs)

    # calculate
    arr = calculate(dset, dset_static)

    # make the plots
    fig = plot(arr, dictArgs["label"])

    filename = f"{dictArgs['outdir']}/template"
    imgbufs = image_handler([fig], dictArgs, filename=filename)

    return imgbufs


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



import numpy as np


def linCI(min, max, ci, *args):
    """
    Returns list of linearly spaced contour intervals from min to max with spacing ci.
    Unline np.arange this max is included IF max = min + ci*N for an integer N.
    """
    if len(args):
        return np.concatenate((np.arange(min, max + ci, ci), linCI(*args)))
    return np.arange(min, max + ci, ci)



def newLims(cur_xlim, cur_ylim, cursor, xlim, ylim, scale_factor):
    cur_xrange = (cur_xlim[1] - cur_xlim[0]) * 0.5
    cur_yrange = (cur_ylim[1] - cur_ylim[0]) * 0.5
    xdata = cursor[0]
    ydata = cursor[1]
    new_xrange = cur_xrange * scale_factor
    new_yrange = cur_yrange * scale_factor
    xdata = min(max(xdata, xlim[0] + new_xrange), xlim[1] - new_xrange)
    xL = max(xlim[0], xdata - new_xrange)
    xR = min(xlim[1], xdata + new_xrange)
    if ylim[1] > ylim[0]:
        ydata = min(max(ydata, ylim[0] + new_yrange), ylim[1] - new_yrange)
        yL = max(ylim[0], ydata - new_yrange)
        yR = min(ylim[1], ydata + new_yrange)
    else:
        ydata = min(max(ydata, ylim[1] - new_yrange), ylim[0] + new_yrange)
        yR = max(ylim[1], ydata + new_yrange)
        yL = min(ylim[0], ydata - new_yrange)
    if (
        xL == cur_xlim[0]
        and xR == cur_xlim[1]
        and yL == cur_ylim[0]
        and yR == cur_ylim[1]
    ):
        return (None, None), (None, None)
    return (xL, xR), (yL, yR)



import numpy


def expandI(a):
    """
    Expands an array by one column, averaging the data to the middle columns and
    extrapolating for the first and last columns. Needed for shifting coordinates
    from centers to corners.
    """
    nj, ni = a.shape
    b = numpy.zeros((nj, ni + 1))
    b[:, 1:-1] = 0.5 * (a[:, :-1] + a[:, 1:])
    b[:, 0] = a[:, 0] + 0.5 * (a[:, 0] - a[:, 1])
    b[:, -1] = a[:, -1] + 0.5 * (a[:, -1] - a[:, -2])
    return b



from .stress_curl import parse, read, calculate, plot, run, parse_and_run

__description__ = "Maps the curl of the stress (N/m**3) acting on the surface ocean"
__ppstreams__ = [
    "ocean_annual/ts",
    "ocean_monthly/ts",
]
__ppvars__ = ["tauuo", "tauvo"]



import numpy
import numpy.matlib
from ..coords import *


def createXYcoords(s, x, y):
    """
    Checks that x and y are appropriate 2D corner coordinates
    and tries to make some if they are not.
    """
    nj, ni = s.shape
    if x is None:
        xCoord = numpy.arange(0.0, ni + 1)
    else:
        xCoord = numpy.ma.filled(x, 0.0)
    if y is None:
        yCoord = numpy.arange(0.0, nj + 1)
    else:
        yCoord = numpy.ma.filled(y, 0.0)

    # Turn coordinates into 2D arrays if 1D arrays were provided
    if len(xCoord.shape) == 1:
        nxy = yCoord.shape
        xCoord = numpy.matlib.repmat(xCoord, nxy[0], 1)
    nxy = xCoord.shape
    if len(yCoord.shape) == 1:
        yCoord = numpy.matlib.repmat(yCoord.T, nxy[-1], 1).T
    if xCoord.shape != yCoord.shape:
        raise Exception("The shape of coordinates are mismatched!")

    # Create corner coordinates from center coordinates is center coordinates were provided
    if xCoord.shape != yCoord.shape:
        raise Exception("The shape of coordinates are mismatched!")
    if s.shape == xCoord.shape:
        xCoord = expandJ(expandI(xCoord))
        yCoord = expandJ(expandI(yCoord))
    return xCoord, yCoord


def createXYlabels(x, y, xlabel, xunits, ylabel, yunits):
    """
    Checks that x and y labels are appropriate and tries to make some if they are not.
    """
    if x is None:
        if xlabel is None:
            xlabel = "i"
        if xunits is None:
            xunits = ""
    else:
        if xlabel is None:
            xlabel = "Longitude"
        # if xunits is None: xunits=u'\u00B0E'
        if xunits is None:
            xunits = r"$\degree$E"
    if y is None:
        if ylabel is None:
            ylabel = "j"
        if yunits is None:
            yunits = ""
    else:
        if ylabel is None:
            ylabel = "Latitude"
        # if yunits is None: yunits=u'\u00B0N'
        if yunits is None:
            yunits = r"$\degree$N"
    return xlabel, xunits, ylabel, yunits



import numpy


def boundaryStats(a):
    """
    Returns the minimum and maximum values of a only on the boundaries of the array.
    """
    amin = numpy.amin(a[0, :])
    amin = min(amin, numpy.amin(a[1:, -1]))
    amin = min(amin, numpy.amin(a[-1, :-1]))
    amin = min(amin, numpy.amin(a[1:-1, 0]))
    amax = numpy.amax(a[0, :])
    amax = max(amax, numpy.amax(a[1:, -1]))
    amax = max(amax, numpy.amax(a[-1, :-1]))
    amax = max(amax, numpy.amax(a[1:-1, 0]))
    return amin, amax



def sectorRanges(sector=None):
    # Should add definitions for tropInd, tropAtl, sAtl, nPac, allPac, allInd, and allAtlantic
    if sector == "nAtl":
        lonRange = (-100.0, 40.0)
        latRange = (-15.0, 80.0)
        hspace = 0.25
        titleOffset = 1.14
    elif sector == "gomex":
        lonRange = (-100.0, -50.0)
        latRange = (5.0, 35.0)
        hspace = 0.25
        titleOffset = 1.14
    elif sector == "tropPac":
        lonRange = (-270.0, -75.0)
        latRange = (-30.0, 30.0)
        hspace = -0.2
        titleOffset = 1.17
    elif sector == "arctic":
        lonRange = (-300, 60)
        latRange = (60.0, 90.0)
        hspace = 0.25
        titleOffset = 1.14
    elif sector == "shACC":
        lonRange = (-300, 60)
        latRange = (-90, -45.0)
        hspace = 0.25
        titleOffset = 1.14
    else:
        lonRange = None
        latRange = None
        hspace = 0.25
        titleOffset = 1.14
    return lonRange, latRange, hspace, titleOffset



# -*- coding: utf-8 -*-
"""m6plot.geoplot - routines to use a higher level mapping package

*** this uses Basemap, which is depreciated; needs to be ported to cartopy ***
"""

__all__ = [
    "sectorRanges",
]

for func in __all__:
    exec(f"from .{func} import *")



from .enso import parse, read, calculate, plot, run, parse_and_run

__description__ = "Plots power spectra and wavelets for NINO regions"
__ppstreams__ = [
    "ocean_monthly_1x1deg/ts",
]
__ppvars__ = ["tos"]



#!/usr/bin/env python3

""" Potential Temperature Drift Diagnostic """

from omlabs.om4parser import default_diag_parser
from omlabs.diags.generic_depth_time_drift import read
from omlabs.diags.generic_depth_time_drift import calculate
from omlabs.diags.generic_depth_time_drift import plot
from omlabs.diags.generic_depth_time_drift import run


def parse(cliargs=None, template=False):
    """Function to capture the user-specified command line options

    Parameters
    ----------
    cliargs : argparse, optional
        Command line options from argparse, by default None
    template : bool, optional
        Return dictionary instead of parser, by default False

    Returns
    -------
        parsed command line arguments
    """

    description = "Diagnostic for layer mean potential temperature drift"

    parser = default_diag_parser(
        description=description,
        template=template,
        exclude=["gridspec", "static", "basin", "obsfile", "hgrid", "topog"],
    )

    parser.add_argument(
        "--description",
        type=str,
        required=False,
        default="Layer-average temperature drift [degC] vs. time",
        help="description string in subtitle",
    )
    parser.add_argument(
        "--varname",
        type=str,
        required=False,
        default="thetao_xyave",
        help="variable name to plot",
    )
    parser.add_argument(
        "--range",
        type=float,
        required=False,
        default=1.0,
        help="min/max data range for contouring",
    )
    parser.add_argument(
        "--interval", type=float, required=False, default=0.1, help="contour interval"
    )

    if template is True:
        return parser.parse_args(None).__dict__
    else:
        return parser.parse_args(cliargs)


def parse_and_run(cliargs=None):
    """Parses command line and runs diagnostic

    Parameters
    ----------
    cliargs : argparse, optional
        command line arguments from upstream instance, by default None

    Returns
    -------
    io.BytesIO
        In-memory image buffer
    """
    args = parse(cliargs)
    args = args.__dict__
    imgbuf = run(args)
    return imgbuf


if __name__ == "__main__":
    parse_and_run()



import matplotlib.pyplot as plt
import numpy
import numpy.matlib
from . import cm
from . import coords
from . import formatting
from . import stats
from . import addStatusBar
from . import addInteractiveCallbacks


def xyplot(
    field,
    x=None,
    y=None,
    area=None,
    xlabel=None,
    xunits=None,
    ylabel=None,
    yunits=None,
    title="",
    suptitle="",
    clim=None,
    colormap=None,
    extend=None,
    centerlabels=False,
    nbins=None,
    landcolor=[0.5, 0.5, 0.5],
    aspect=[16, 9],
    resolution=576,
    axis=None,
    sigma=2.0,
    ignore=None,
    save=None,
    debug=False,
    show=False,
    interactive=False,
    logscale=False,
):
    """
    Renders plot of scalar field, field(x,y).

    Arguments:
    field        Scalar 2D array to be plotted.
    x            x coordinate (1D or 2D array). If x is the same size as field then x is treated as
                 the cell center coordinates.
    y            y coordinate (1D or 2D array). If x is the same size as field then y is treated as
                 the cell center coordinates.
    area         2D array of cell areas (used for statistics). Default None.
    xlabel       The label for the x axis. Default 'Longitude'.
    xunits       The units for the x axis. Default 'degrees E'.
    ylabel       The label for the y axis. Default 'Latitude'.
    yunits       The units for the y axis. Default 'degrees N'.
    title        The title to place at the top of the panel. Default ''.
    suptitle     The super-title to place at the top of the figure. Default ''.
    clim         A tuple of (min,max) color range OR a list of contour levels. Default None.
    sigma         Sigma range for difference plot autocolor levels. Default is to span a 2. sigma range
    colormap     The name of the colormap to use. Default None.
    extend       Can be one of 'both', 'neither', 'max', 'min'. Default None.
    centerlabels If True, will move the colorbar labels to the middle of the interval. Default False.
    nbins        The number of colors levels (used is clim is missing or only specifies the color range).
    landcolor    An rgb tuple to use for the color of land (no data). Default [.5,.5,.5].
    aspect       The aspect ratio of the figure, given as a tuple (W,H). Default [16,9].
    resolution   The vertical resolution of the figure given in pixels. Default 720.
    axis         The axis handle to plot to. Default None.
    ignore       A value to use as no-data (NaN). Default None.
    save         Name of file to save figure in. Default None.
    debug        If true, report stuff for debugging. Default False.
    show         If true, causes the figure to appear on screen. Used for testing. Default False.
    interactive  If true, adds interactive features such as zoom, close and cursor. Default False.
    logscale     If true, use logaritmic coloring scheme. Default False.
    """

    # set visual backend
    if interactive is False:
        plt.switch_backend("Agg")

    c = cm.dunne_pm()
    c = cm.dunne_rainbow()

    # Create coordinates if not provided
    xlabel, xunits, ylabel, yunits = formatting.createXYlabels(
        x, y, xlabel, xunits, ylabel, yunits
    )
    if debug:
        print("x,y label/units=", xlabel, xunits, ylabel, yunits)
    xCoord, yCoord = formatting.createXYcoords(field, x, y)

    # Diagnose statistics
    if ignore is not None:
        maskedField = numpy.ma.masked_array(field, mask=[field == ignore])
    else:
        maskedField = field.copy()
    sMin, sMax, sMean, sStd, sRMS = stats.calc(maskedField, area, debug=debug)
    xLims = coords.boundaryStats(xCoord)
    yLims = coords.boundaryStats(yCoord)

    # Choose colormap
    if nbins is None and (clim is None or len(clim) == 2):
        nbins = 35
    if colormap is None:
        colormap = chooseColorMap(sMin, sMax)
    if clim is None and sStd > 0:
        cmap, norm, extend = cm.chooseColorLevels(
            sMean - sigma * sStd,
            sMean + sigma * sStd,
            colormap,
            clim=clim,
            nbins=nbins,
            extend=extend,
            logscale=logscale,
        )
    else:
        cmap, norm, extend = cm.chooseColorLevels(
            sMin,
            sMax,
            colormap,
            clim=clim,
            nbins=nbins,
            extend=extend,
            logscale=logscale,
        )

    if axis is None:
        formatting.setFigureSize(aspect, resolution, debug=debug)
        # plt.gcf().subplots_adjust(left=.08, right=.99, wspace=0, bottom=.09, top=.9, hspace=0)
        axis = plt.gca()
    plt.pcolormesh(xCoord, yCoord, maskedField, cmap=cmap, norm=norm)
    if interactive:
        addStatusBar(xCoord, yCoord, maskedField)
    cb = plt.colorbar(fraction=0.08, pad=0.02, extend=extend)
    if centerlabels and len(clim) > 2:
        cb.set_ticks(0.5 * (clim[:-1] + clim[1:]))
    elif clim is not None and len(clim) > 2:
        cb.set_ticks(clim)
    axis.set_facecolor(landcolor)
    plt.xlim(xLims)
    plt.ylim(yLims)
    axis.annotate(
        "max=%.5g\nmin=%.5g" % (sMax, sMin),
        xy=(0.0, 1.01),
        xycoords="axes fraction",
        verticalalignment="bottom",
        fontsize=10,
    )
    if area is not None:
        axis.annotate(
            "mean=%.5g\nrms=%.5g" % (sMean, sRMS),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="right",
            fontsize=10,
        )
        axis.annotate(
            " sd=%.5g\n" % (sStd),
            xy=(1.0, 1.01),
            xycoords="axes fraction",
            verticalalignment="bottom",
            horizontalalignment="left",
            fontsize=10,
        )
    if len(xlabel + xunits) > 0:
        plt.xlabel(formatting.label(xlabel, xunits))
    if len(ylabel + yunits) > 0:
        plt.ylabel(formatting.label(ylabel, yunits))
    if len(title) > 0:
        plt.title(title)
    if len(suptitle) > 0:
        plt.suptitle(suptitle)

    if save is not None:
        plt.savefig(save)
    if interactive:
        addInteractiveCallbacks()
    if show:
        plt.show(block=False)

    return plt.gcf()



import numpy


def expand(a):
    """
    Expands a vector by one element, averaging the data to the middle columns and
    extrapolating for the first and last rows. Needed for shifting coordinates
    from centers to corners.
    """
    b = numpy.zeros((len(a) + 1))
    b[1:-1] = 0.5 * (a[:-1] + a[1:])
    b[0] = a[0] + 0.5 * (a[0] - a[1])
    b[-1] = a[-1] + 0.5 * (a[-1] - a[-2])
    return b



from .acc_timeseries import parse, read, calculate, plot, run, parse_and_run

__description__ = (
    "Plots time series of total volume transport through the Drake Passage"
)
__ppstreams__ = [
    "ocean_month_z_d2_refined/av",
    "ocean_monthly_z_d2/ts",
    "ocean_annual_z/av",
]
__ppvars__ = ["umo"]



import math
import numpy


def calc(s, area, s2=None, debug=False):
    """
    Calculates mean, standard deviation and root-mean-square of s.
    """
    sMin = numpy.ma.min(s)
    sMax = numpy.ma.max(s)
    if area is None:
        return sMin, sMax, None, None, None
    weight = area.copy()
    if debug:
        print("stats: sum(area) =", numpy.ma.sum(weight))
    if not numpy.ma.getmask(s).any() == numpy.ma.nomask:
        weight[s.mask] = 0.0
    sumArea = numpy.ma.sum(weight)
    if debug:
        print("stats: sum(area) =", sumArea, "after masking")
    if debug:
        print("stats: sum(s) =", numpy.ma.sum(s))
    if debug:
        print("stats: sum(area*s) =", numpy.ma.sum(weight * s))
    mean = numpy.ma.sum(weight * s) / sumArea
    std = math.sqrt(numpy.ma.sum(weight * ((s - mean) ** 2)) / sumArea)
    rms = math.sqrt(numpy.ma.sum(weight * (s ** 2)) / sumArea)
    if debug:
        print("stats: mean(s) =", mean)
    if debug:
        print("stats: std(s) =", std)
    if debug:
        print("stats: rms(s) =", rms)
    return sMin, sMax, mean, std, rms



def regionalMasking(field, yCoord, xCoord, latRange, lonRange):
    maskedField = field.copy()
    maskedField = numpy.ma.masked_where(
        numpy.less(yCoord[0:-1, 0:-1], latRange[0]), maskedField
    )
    maskedField = numpy.ma.masked_where(
        numpy.greater(yCoord[1::, 1::], latRange[1]), maskedField
    )
    maskedField = numpy.ma.masked_where(
        numpy.less(xCoord[0:-1, 0:-1], lonRange[0]), maskedField
    )
    maskedField = numpy.ma.masked_where(
        numpy.greater(xCoord[1::, 1::], lonRange[1]), maskedField
    )
    return maskedField



import numpy
import math


def corr(s1, s2, area):
    """
    Calculates the correlation coefficient between s1 and s2, assuming s1 and s2 have
    not mean. That is s1 = S - mean(S), etc.
    """
    weight = area.copy()
    if not numpy.ma.getmask(s1).any() == numpy.ma.nomask:
        weight[s1.mask] = 0.0
    sumArea = numpy.ma.sum(weight)
    v1 = numpy.ma.sum(weight * (s1 ** 2)) / sumArea
    v2 = numpy.ma.sum(weight * (s2 ** 2)) / sumArea
    if v1 == 0 or v2 == 0:
        return numpy.NaN
    rxy = numpy.ma.sum(weight * (s1 * s2)) / sumArea / math.sqrt(v1 * v2)
    return rxy



from .generic_depth_time_drift import parse, read, calculate, plot, run, parse_and_run

__description__ = "Generic layer-average drift diagnostic"
__ppstreams__ = [
    "ocean_annual_z/ts",
]
__ppvars__ = ["thetao_xyave", "so_xyave"]



# need a data_read, data_sel, data_reduce

import argparse
import calendar
import glob
import io
import os
import pathlib
import signal
import sys
import tarfile as tf
import warnings
from datetime import datetime

import intake
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pkg_resources as pkgr
import scipy
import xarray as xr
from cmip_basins import generate_basin_codes
from packaging import version
from xgcm import Grid

try:
    import xesmf as xe
except ModuleNotFoundError as exc:
    warnings.warn("Unable to import xesmf. Some functionality may not work.")

try:
    from omlabs.helpers import try_variable_from_list
except ImportError:
    # DORA mode, works without install.
    # reads from current directory
    from helpers import try_variable_from_list

from static_downsampler.static import (
    extend_supergrid_array,
    subsample_supergrid,
    sum_on_supergrid,
)

possible_names = {}
possible_names["lon"] = ["lon", "LON", "longitude", "LONGITUDE"]
possible_names["lat"] = ["lat", "LAT", "latitude", "LATITUDE"]
possible_names["time"] = ["time", "TIME", "latitude"]
possible_names["depth"] = ["z_l", "depth", "DEPTH"]
possible_names["interfaces"] = ["z_i"]


class DefaultDictParser(argparse.ArgumentParser):
    """argparse extention that bypasses error and returns a dict of defaults"""

    def error(self, message):
        actions = self.__dict__["_actions"]
        defaults = {}
        for act in actions[1::]:
            defaults[act.__dict__["dest"]] = act.__dict__["default"]
        return defaults


def date_range(ds, ref_time="1970-01-01T00:00:00Z"):
    """Returns a tuple of start year and end year from xarray dataset

    Parameters
    ----------
    ds : xarray.Dataset
        Input dataset

    Returns
    -------
    tuple
        (start year, end year) for time dimension of the dataset
    """

    if "time_bnds" in list(ds.variables):

        # Xarray decodes bounds times relative to the epoch and
        # returns a numpy timedelta object in some instances
        # instead of a cftime datetime object. Manual decoding
        # and shifting may be necessary

        if isinstance(ds["time_bnds"].values[0][0], np.timedelta64):

            # When opening multi-file datasets with open_mfdataset(),
            # xarray strips out the calendar encoding. Since bounds
            # are computed differently to begin with, fall back to
            # another FMS generated time variable to get the calendar
            # base date.

            if "units" in ds["time"].encoding.keys():
                base_time = ds["time"].encoding["units"]
            elif "units" in ds["average_T1"].encoding.keys():
                base_time = ds["average_T1"].encoding["units"]
            else:
                base_time = None

            if base_time is not None:
                base_time = base_time.split(" ")[2:4]
                base_time = np.datetime64(f"{base_time[0]}T{base_time[1]}Z")
                offset = base_time - np.datetime64(ref_time)

                t0 = ds["time_bnds"].values[0][0] + offset
                t0 = datetime.fromtimestamp(int(np.ceil(int(t0) * 1.0e-9)))
                t0 = tuple(t0.timetuple())
                # if start bound is Dec-31, advance to next year
                t0 = (t0[0] + 1) if (t0[1:3] == (12, 31)) else t0[0]

                t1 = ds["time_bnds"].values[-1][-1] + offset
                t1 = datetime.fromtimestamp(int(np.ceil(int(t1) * 1.0e-9)))
                t1 = tuple(t1.timetuple())
                # if end bound is Jan-1, fall back to previous year
                t1 = (t1[0] - 1) if (t1[1:3] == (1, 1)) else t1[0]

            else:
                # return very obvious incorrect dates to alert the
                # user that the inferred time range failed
                t0 = 9999
                t1 = 9999

        else:
            t0 = tuple(ds["time_bnds"].values[0][0].timetuple())[0]

            # if end bound is Jan-1, fall back to previous year
            t1 = tuple(ds["time_bnds"].values[-1][-1].timetuple())
            t1 = (t1[0] - 1) if (t1[1:3] == (1, 1)) else t1[0]

    else:
        t0 = int(ds["time"].isel({"time": 0}).dt.strftime("%Y"))
        t1 = int(ds["time"].isel({"time": -1}).dt.strftime("%Y"))

    return (t0, t1)


def discover_ts_dir(path, default="ts/monthly"):
    """Find the directory with the longest timeseries chunk

    Parameters
    ----------
    path : str, path-like
        Path to top-level (.../pp) post-processing directory
    default : str, optional
        Time resolution to scan, by default "ts/monthly"

    Returns
    -------
    str, path-like
        Full path to timeseries directory with the longest chunks
    """

    # combine root pp path with the default chunk
    path = f"{path}/{default}"
    path = fixdir(path)

    # get a list of subdirectories
    subdirs = [f.path for f in os.scandir(path) if f.is_dir()]
    subdirs = [dirname.replace(f"{path}/", "") for dirname in subdirs]

    # chomp the last part of the path to obtain the numerical chunk length
    subdirs = [int(dirname.replace("yr", "")) for dirname in subdirs]

    # reconstruct and return the path
    return_path = f"{path}/{max(subdirs)}yr/"
    return fixdir(return_path)


def extract_from_tar(tar, member):
    """Loads Xarray DataSet in memory from a file contained inside a tar file

    Parameters
    ----------
    tar : tarfile.TarFile
        TarFile object generated through tarfile.open('file')
    member : str
        Name of dataset to extract to memory

    Returns
    -------
    xarray.DataSet
        In-memory Xarray dataset
    """

    if member not in tar.getnames():
        member = "./" + member
    f = tar.extractfile(member)
    data = f.read()

    # the line below is retained for NetCDF4 library reference
    # dataset = netCDF4.Dataset("in-mem-file", mode="r", memory=data)

    dataset = xr.open_dataset(data)

    return dataset


def fixdir(path):
    """Ensures a path string does not contain a double-slash

    Parameters
    ----------
    path : str, path-like
        string containing a path

    Returns
    -------
    string with double slashes removed

    """
    return path.replace("//", "/")


def generate_basin_masks(basin_code, basin=None):
    """Returns 2-D array mask (1s/0s) for common pre-defined
    basins and regions.

    Parameters
    ----------
    basin_code : numpy.ndarray
        2-dimensional array of CMIP-convention basin codes
    basin : str or int, optional
        Name of basin to calculate. Options are "atlantic_arctic"
        and "indo_pacific". An integer basin code may also be
        passed. By default None

    Returns
    -------
    numpy.ndarray
        Basin mask of 1s and 0s.
    """
    mask = basin_code * 0
    if basin == "atlantic_arctic":
        mask[
            (basin_code == 2)
            | (basin_code == 4)
            | (basin_code == 6)
            | (basin_code == 7)
            | (basin_code == 8)
        ] = 1.0
    elif basin == "indo_pacific":
        mask[(basin_code == 3) | (basin_code == 5)] = 1.0
    elif isinstance(basin, int):
        mask[(basin_code == basin)] = 1.0
    else:
        mask[(basin_code >= 1)] = 1.0
    return mask


def image_handler(figs, dictArgs, dpi=100, filename="./figure"):
    """Generic OM4Labs image handler. Depending on the framework mode,
    this handler either saves a matplotlib figure handle to disk or
    returns an in-memory image buffer

    Parameters
    ----------
    figs : matplotlib.Figure or list
        Matplotlib figure handle or list of figure handles
    dictArgs : dict
        Dictionary of parsed command-line options
    dpi : int, optional
        Figure resolution in dots per inch, by default 100
    filename : str, optional
        Figure filename, by default "./figure"

    Returns
    -------
    io.BytesIO
        In-memory image buffers
    """

    # Set default dpi
    dpi = dictArgs["dpi"] if "dpi" in dictArgs.keys() else dpi

    imgbufs = []
    numfigs = len(figs)

    # test if output directory exists
    if dictArgs["outdir"] != "./":
        pathlib.Path(dictArgs["outdir"]).mkdir(parents=True, exist_ok=True)

    if not isinstance(filename, list):
        filename = [filename]

    assert (
        len(filename) == numfigs
    ), "Number of figure handles and file names do not match."

    if dictArgs["interactive"] is True:
        plt.show()

    else:
        for n, fig in enumerate(figs):
            if dictArgs["format"] == "stream":
                imgbuf = io.BytesIO()
                fig.savefig(imgbuf, format="png", dpi=dpi, bbox_inches="tight")
                imgbufs.append(imgbuf)
            else:
                fig.savefig(
                    f"{filename[n]}.png",
                    format=dictArgs["format"],
                    dpi=dpi,
                    bbox_inches="tight",
                )

    return imgbufs


def infer_and_assign_coord(ds, da, coordname):
    """infer what the coord name is and assign it to dataarray"""
    assigned_coordname = try_variable_from_list(
        list(ds.variables), possible_names[coordname]
    )
    if (assigned_coordname is not None) and (assigned_coordname in da.dims):
        da = da.rename({assigned_coordname: f"assigned_{coordname}"})
    return da


def read_data(ds, possible_variable_names):
    """read data from one file"""

    # find the appropriate variable names
    varname = try_variable_from_list(list(ds.variables), possible_variable_names)
    if varname is None:
        raise ValueError(f"no suitable variable found in dataset")

    da = ds[varname]
    da = infer_and_assign_coord(ds, da, "lon")
    da = infer_and_assign_coord(ds, da, "lat")
    da = infer_and_assign_coord(ds, da, "time")
    da = infer_and_assign_coord(ds, da, "depth")
    da = infer_and_assign_coord(ds, da, "interfaces")

    return da


def standard_grid_cell_area(lat, lon, rE=6371.0e3):
    """computes the cell area for a standard spherical grid"""

    warnings.warn(
        "standard_grid_cell_area is deprecated, use compute_area_regular_grid",
        DeprecationWarning,
    )

    dLat = lat[1] - lat[0]
    dLon = lon[1] - lon[0]
    area = np.empty((len(lat), len(lon)))
    for j in range(0, len(lat)):
        for i in range(0, len(lon)):
            lon1 = lon[i] + dLon / 2
            lon0 = lon[i] - dLon / 2
            lat1 = lat[j] + dLat / 2
            lat0 = lat[j] - dLat / 2
            area[j, i] = (
                (np.pi / 180.0)
                * rE
                * rE
                * np.abs(np.sin(np.radians(lat0)) - np.sin(np.radians(lat1)))
                * np.abs(lon0 - lon1)
            )
    return area


def standardize_longitude(dset, lon_coord, start_lon=0.0):
    """Processes a dataset so that the longitude ranges from
    0 to 360 degrees and that it is montonically increasing

    Parameters
    ----------
    dset : xarray.Dataset
        Input dataset
    lon_coord : str
        Name of longitude coordinate
    start_lon : float
        Starting longitude ranging from -180. to 0,
        by default 0.

    Returns
    ----------
    xarray.Dataset

    """

    def _is_in_range(x, start_lon, end_lon):
        return np.all(x < end_lon) & np.all(x > start_lon)

    def _is_monotonic(x):
        return np.all(np.diff(x) > 0)

    # test to see if the starting longitude is between 0 and 360
    if not ((start_lon >= -180.0) & (start_lon <= 0.0)):
        raise ValueError(
            "Starting longitude must be between -180 and 0 to ensure monotonicity"
        )

    # determine the ending longitude
    end_lon = start_lon + 360.0

    # copy the longitude dimension for manipulation
    lon = dset[lon_coord]

    # ensure the values are in the specified range
    if not _is_in_range(lon, start_lon, end_lon):
        lon = xr.where(lon > end_lon, lon - 360.0, lon)
        lon = xr.where(lon < start_lon, 360.0 + lon, lon)
        assert _is_in_range(
            lon, start_lon, end_lon
        ), "Unable to standardize longitude range"
        dset = dset.assign_coords({lon_coord: lon})

    # we can roll the dataset to get a monotonically ordered
    # dimension for a one-dimensional x-dim, but best not to
    # do this if we have a two-dimensionalx-dim

    if len(lon.shape) == 1:
        if not _is_monotonic(lon):
            roll_delta = np.argmin(np.array(lon))
            dset = dset.roll({lon_coord: -roll_delta}, roll_coords=True)
            assert _is_monotonic(dset[lon_coord]), "Longitude is not monotonic"

    return dset


def subset_data(da, coordname, subset):
    """subset (float or slice) dataarray along coord"""
    if coordname in da.coords:
        da = da.sel({coordname: subset})
    return da


def simple_average(da, coordname):
    """average"""
    if coordname in da.coords:
        da = da.mean(dim=coordname)
    return da


def copy_coordinates(da1, da2, coords):
    """copy coordinates of da1 into da2"""
    for coord in coords:
        da2[coord] = da1[coord]

    return da2


def compute_area_regular_grid(ds, Rearth=6378e3):
    """compute the cells area on a regular grid"""

    rfac = 2 * np.pi * Rearth / 360

    up = {"bnds": 1}
    down = {"bnds": 0}
    if "time" in ds["lon_bnds"].dims:
        up.update({"time": 0})
        down.update({"time": 0})

    dx1d = rfac * (ds["lon_bnds"].isel(up) - ds["lon_bnds"].isel(down))
    dy1d = rfac * (ds["lat_bnds"].isel(up) - ds["lat_bnds"].isel(down))

    dx2d, dy2d = np.meshgrid(dx1d, dy1d)
    _, lat2d = np.meshgrid(ds["lon"].values, ds["lat"].values)

    dx = dx2d * np.cos(2 * np.pi * lat2d / 360)
    dy = dy2d
    area = dx * dy
    return area


def is_symmetric(dset, x_center="xh", x_corner="xq", y_center="yh", y_corner="yq"):
    """Determines if ocean model output is on a symmetric grid

    Parameters
    ----------
    dset : [type]
        [description]
    x_center : str, optional
        Name of x-cell centers dimension, by default "yxh"
    x_corner : str, optional
        Name of x-cell corners dimension, by default "xq"
    y_center : str, optional
        Name of y-cell centers dimension, by default "yh"
    y_corner : str, optional
        Name of y-cell corners dimension, by default "yq"

    Returns
    -------
    bool
        True if grid is symmetric
    """

    if (len(dset[x_corner]) == len(dset[x_center])) and (
        len(dset[y_corner]) == len(dset[y_center])
    ):
        out = False
    elif (len(dset[x_corner]) == len(dset[x_center]) + 1) and (
        len(dset[y_corner]) == len(dset[y_center]) + 1
    ):
        out = True
    else:
        raise ValueError("unsupported combination of coordinates")
    return out


def grid_from_supergrid(ds, point_type="t", outputgrid="nonsymetric"):
    """Subsample super grid to obtain geolon, geolat, and cell area

    Parameters
    ----------
    ds : xarray.Dataset
        Input dataset containing variables from the supergrid
    point_type : str, optional
        Requested grid type of t|q|u|v, by default "t"
    outputgrid : str, optional
        Either "symetric" or "nonsymetric", default is "nonsymetric"

    Returns
    -------
    geolat : xarray.DataArray
        2-dimensional Earth-centric latitude coordinates
    geolon : xarray.DataArray
        2-dimensional Earth-centric longitude coordinates
    area : xarray.DataArray
        Array of cell areas with dimension (geolat,geolon)
    """

    geolat = subsample_supergrid(ds, "y", point_type, outputgrid=outputgrid)
    geolon = subsample_supergrid(ds, "x", point_type, outputgrid=outputgrid)
    area = sum_on_supergrid(ds, "area", point_type, outputgrid=outputgrid)

    return geolat, geolon, area


def horizontal_grid(
    dictArgs=None,
    point_type="t",
    coords=None,
    outputgrid="nonsymetric",
    output_type="xarray",
):
    """Returns horizontal grid parameters based on the values of the CLI
    arguments and the presence of intake catalogs.

    The requested grid can either tracer points (t), corner points (q),
    zonal velocity points (u), or meridional velocity points (v).

    The corresponding basin mask for the grid is also returned.

    The nominal x and y 1-D coordinates are provided.  Since these are
    non-physical,however, they should only be used for plotting purposes.

    If `dictArgs` is omitted, the function returns a standard 1x1
    spherical grid.

    Parameters
    ----------
    dictArgs : dict, optional
        dictionary of arguments obtained from the CLI parser, by default None
    point_type : str, optional
        Requested grid type of t|q|u|v, by default "t"
    coords : tuple, optional
        target xarray coordinates
    outputgrid : str, optional
        Either "symetric" or "nonsymetric", default is "nonsymetric"
    output_type : str, optional
        Specify output format of either "xarray" or "numpy", by default "xarray"

    Returns
    -------
    xarray.Dataset or tuple
        Arrays of geolat, geolon, nominal_x, nominal_y, area, and basin
    """
    point_type = point_type.upper()

    # if verbose if present in dictArgs that was generated by the parser,
    # that value takes precedence over the kwarg version
    verbose = dictArgs["verbose"] if "verbose" in dictArgs else False
    basin_file = dictArgs["basin"] if "basin" in dictArgs else None

    if dictArgs is None:
        x = np.arange(0.5, 360.5, 1.0)
        y = np.arange(-89.5, 90.5, 1.0)
        area = standard_grid_cell_area(y, x)
        geolon, geolat = np.meshgrid(x, y)
        geolon = xr.DataArray(geolon, dims=("y", "x"), coords={"y": y, "x": x})
        geolat = xr.DataArray(geolat, dims=("y", "x"), coords={"y": y, "x": x})
        area = xr.DataArray(area, dims=("y", "x"), coords={"y": y, "x": x})
        nominal_x = geolon[geolon.dims[-1]]
        nominal_y = geolat[geolat.dims[-2]]

    elif dictArgs["hgrid"] is not None:
        if verbose:
            print("Using optional hgrid file for horizontal grid.")
        ds = xr.open_dataset(dictArgs["hgrid"])
        geolat, geolon, area = grid_from_supergrid(
            ds, point_type, outputgrid=outputgrid
        )
        wet = infer_wet_mask(dictArgs, coords=coords, point_type=point_type)
        warnings.warn(
            "Inferring wet mask from topography. Consider using ocean_static.nc"
        )

    elif dictArgs["static"] is not None:
        if verbose:
            print("Using optional static file for horizontal grid.")

        ds = xr.open_dataset(dictArgs["static"])

        if point_type == "T":
            geolat = ds["geolat"]
            geolon = ds["geolon"]
            wet = ds["wet"]
            area = ds["areacello"]

        elif point_type == "U":
            geolat = ds["geolat_u"]
            geolon = ds["geolon_u"]
            wet = ds["wet_u"]
            area = ds["areacello_cu"]

        elif point_type == "V":
            geolat = ds["geolat_v"]
            geolon = ds["geolon_v"]
            wet = ds["wet_v"]
            area = ds["areacello_cv"]

        else:
            raise ValueError("Unknown point type. Must be T, U, or V")

    elif dictArgs["gridspec"] is not None:
        if verbose:
            print("Using optional gridspec tar file for horizontal grid.")
        tar = tf.open(dictArgs["gridspec"])
        ds = extract_from_tar(tar, "ocean_hgrid.nc")
        geolat, geolon, area = grid_from_supergrid(
            ds, point_type, outputgrid=outputgrid
        )
        wet = infer_wet_mask(dictArgs, coords=coords, point_type=point_type)
        warnings.warn(
            "Inferring wet mask from topography. Consider using ocean_static.nc"
        )

    elif dictArgs["platform"] is not None and dictArgs["config"] is not None:
        if verbose:
            print(
                f"Using {dictArgs['platform']} {dictArgs['config']} intake catalog for horizontal grid."
            )
        cat = open_intake_catalog(dictArgs["platform"], dictArgs["config"])
        ds = cat["ocean_hgrid"].to_dask()
        geolat, geolon, area = grid_from_supergrid(
            ds, point_type, outputgrid=outputgrid
        )
        wet = infer_wet_mask(dictArgs, coords=coords, point_type=point_type)
        warnings.warn(
            "Inferring wet mask from topography. Consider using ocean_static.nc"
        )

    result = xr.Dataset()
    result["geolat"] = geolat
    result["geolon"] = geolon
    result["area"] = area
    result["wet"] = wet

    # nominal coordinates
    result["nominal_x"] = result.geolon.max(axis=-2)
    result["nominal_y"] = result.geolat.max(axis=-1)

    if result["nominal_y"].min() >= 0:
        warnings.warn("Nominal y latitude is positive definite. May be incorrect.")

    if result["nominal_x"].max() > 360:
        warnings.warn("Nominal x longitude > 360. May be incorrect.")

    # -- process basin codes while we are here
    if basin_file is not None:
        if verbose:
            print("Using optional file for basin code specification.")
        ds = xr.open_dataset(dictArgs["hgrid"])
        result["basin"] = ds.basin
    else:
        result["basin"] = generate_basin_codes(result, lon="geolon", lat="geolat")
    result["basin"] = result.basin.fillna(0.0)

    if output_type == "numpy":
        geolat = np.array(result.geolat.to_masked_array())
        geolon = np.array(result.geolon.to_masked_array())
        nominal_x = np.array(result.nominal_x.to_masked_array())
        nominal_y = np.array(result.nominal_y.to_masked_array())
        area = np.array(result.area.to_masked_array())
        basin = np.array(result.basin.to_masked_array())
        result = (geolat, geolon, nominal_x, nominal_y, area, basin)

    return result


def open_intake_catalog(platform, config):
    """Returns an Intake catalog for a specified platform and config

    Uses the package resources included in the omlabs distribution
    to determine the directory of the intake catalogs, unless it
    is overridden by the "OM4LABS_CATALOG_DIR" environment var.

    Parameters
    ----------
    platform : str
        Site description, e.g. "gfdl", "orion", "testing"
    config : str
        Model configuration, e.g. "OM4p5", "OM4p25"

    Returns
    -------
    intake.catalog.Catalog
        Intake catalog corresponding to specified platform/config
    """

    catalog_str = f"{config}_catalog_{platform}.yml"

    if "OM4LABS_CATALOG_DIR" in os.environ.keys():
        catfile = f"{os.environ['OM4LABS_CATALOG_DIR']}/{catalog_str}"
    else:
        catfile = pkgr.resource_filename("omlabs", f"catalogs/{catalog_str}")

    cat = intake.open_catalog(catfile)

    return cat


def read_topography(dictArgs, coords=None, point_type="t"):
    """Returns topography field based on the values of the CLI
    arguments and the presence of intake catalogs.

    Parameters
    ----------
    dictArgs : dict, optional
        dictionary of arguments obtained from the CLI parser, by default None
    coords : tuple, optional
        target xarray coordinates
    point_type : str, optional
        Requested grid type of t|q|u|v, by default "t"

    Returns
    -------
    xarray.DataArray
        topography array
    """

    point_type = point_type.upper()

    verbose = dictArgs["verbose"] if "verbose" in dictArgs else False

    if dictArgs["topog"] is not None:
        if verbose:
            print("Using optional topg file for depth field")
        ds = xr.open_dataset(dictArgs["topog"])

    elif dictArgs["static"] is not None:
        if verbose:
            print("Using optional static file for depth field.")
        ds = xr.open_dataset(dictArgs["static"])

    elif dictArgs["gridspec"] is not None:
        if verbose:
            print("Using optional gridspec tar file for depth field.")
        tar = tf.open(dictArgs["gridspec"])
        ds = extract_from_tar(tar, "ocean_topog.nc")

    elif dictArgs["platform"] is not None and dictArgs["config"] is not None:
        if verbose:
            print(
                f"Using {dictArgs['platform']} {dictArgs['config']} intake catalog for depth field."
            )
        cat = open_intake_catalog(dictArgs["platform"], dictArgs["config"])
        ds = cat["topog"].to_dask()

    if "deptho" in list(ds.variables):
        depth = ds.deptho
    elif "depth" in list(ds.variables):
        depth = ds.depth

    coords = xr.Dataset(coords)
    xedge = "outer" if (len(coords.xq) == len(coords.xh) + 1) else "right"
    yedge = "outer" if (len(coords.yq) == len(coords.yh) + 1) else "right"
    out_grid = Grid(
        coords,
        coords={"X": {"center": "xh", xedge: "xq"}, "Y": {"center": "yh", yedge: "yq"}},
        periodic=["X"],
    )

    if point_type == "V":
        depth = out_grid.interp(depth, "Y", boundary="fill")
    elif point_type == "U":
        depth = out_grid.interp(depth, "X", boundary="fill")

    return depth


def infer_wet_mask(dictArgs, coords=None, point_type="t"):
    """Infers the model's wet mask based on the model topography

    Parameters
    ----------
    dictArgs : dict, optional
        dictionary of arguments obtained from the CLI parser, by default None
    coords : tuple, optional
        target xarray coordinates
    point_type : str, optional
        Requested grid type of t|q|u|v, by default "t"

    Returns
    -------
    xarray.DataArray
        wet mask of ocean=1, land=0
    """
    depth = read_topography(dictArgs, coords=coords, point_type=point_type)
    depth = xr.where(depth.isnull(), 0.0, depth)
    return xr.where(depth > 0.0, 1.0, 0.0)


def annual_cycle(ds, var):
    """Compute annual cycle climatology"""
    # Make a DataArray with the number of days in each month, size = len(time)
    if hasattr(ds.time, "calendar"):
        cal = ds.time.calendar
    elif hasattr(ds.time, "calendar_type"):
        cal = ds.time.calendar_type.lower()
    else:
        cal = "standard"

    if cal.lower() in ["noleap", "365day"]:
        # always calculate days in month based on year 1 (non-leap year)
        month_length = [calendar.monthrange(1, x.month)[1] for x in ds.time.to_index()]
    else:
        # use real year/month combo to calculate days in month
        month_length = [
            calendar.monthrange(x.year, x.month)[1] for x in ds.time.to_index()
        ]

    month_length = xr.DataArray(month_length, coords=[ds.time], name="month_length")

    # Calculate the weights by grouping by 'time.season'.
    # Conversion to float type ('astype(float)') only necessary for Python 2.x
    weights = (
        month_length.groupby("time.month") / month_length.groupby("time.month").sum()
    )

    # Test that the sum of the weights for each season is 1.0
    np.testing.assert_allclose(weights.groupby("time.month").sum().values, np.ones(12))

    # Calculate the weighted average
    ds_weighted = (ds[var] * weights).groupby("time.month").sum(dim="time")

    return ds_weighted


def add_matrix_NaNs(regridder):
    """Helper function to set masked points to NaN instead of zero"""
    X = regridder.weights
    M = scipy.sparse.csr_matrix(X)
    num_nonzeros = np.diff(M.indptr)
    M[num_nonzeros == 0, 0] = np.NaN
    regridder.weights = scipy.sparse.coo_matrix(M)
    return regridder


def curv_to_curv(src, dst, reuse_weights=False):
    regridder = xe.Regridder(src, dst, "bilinear", reuse_weights=reuse_weights)
    if version.parse(xe.__version__) < version.parse("0.4.0"):
        regridder = add_matrix_NaNs(regridder)
    return regridder(src)



from .stratification import parse, read, calculate, plot, run, parse_and_run

__description__ = "Evaluate stratification parameters in z-coordinates"
__ppstreams__ = [
    "ocean_annual_z_1x1deg/av",
]
__ppvars__ = ["thetao"]



from .template import parse, read, calculate, plot, run, parse_and_run

__description__ = "Template for creatig a new diagnostic"
__ppstreams__ = [
    "ocean_annual_z/av",
]
__ppvars__ = ["some_variable"]



import matplotlib.pyplot as plt
import numpy
import numpy.matlib
from . import cm
from . import coords
from . import formatting
from . import geoplot
from . import stats
from . import addStatusBar
from . import addInteractiveCallbacks


def xycompare(
    field1,
    field2,
    x=None,
    y=None,
    area=None,
    xlabel=None,
    xunits=None,
    ylabel=None,
    yunits=None,
    title1="",
    title2="",
    title3="A - B",
    addplabel=True,
    suptitle="",
    clim=None,
    colormap=None,
    extend=None,
    centerlabels=False,
    dlim=None,
    dcolormap=None,
    dextend=None,
    centerdlabels=False,
    nbins=None,
    landcolor=[0.5, 0.5, 0.5],
    sector=None,
    webversion=False,
    aspect=None,
    resolution=None,
    axis=None,
    npanels=3,
    sigma=2.0,
    ignore=None,
    save=None,
    debug=False,
    show=False,
    interactive=False,
):
    """
    Renders n-panel plot of two scalar fields, field1(x,y) and field2(x,y).

    Arguments:
    field1        Scalar 2D array to be plotted and compared to field2.
    field2        Scalar 2D array to be plotted and compared to field1.
    x             x coordinate (1D or 2D array). If x is the same size as field then x is treated as
                  the cell center coordinates.
    y             y coordinate (1D or 2D array). If x is the same size as field then y is treated as
                  the cell center coordinates.
    area          2D array of cell areas (used for statistics). Default None.
    xlabel        The label for the x axis. Default 'Longitude'.
    xunits        The units for the x axis. Default 'degrees E'.
    ylabel        The label for the y axis. Default 'Latitude'.
    yunits        The units for the y axis. Default 'degrees N'.
    title1        The title to place at the top of panel 1. Default ''.
    title2        The title to place at the top of panel 1. Default ''.
    title3        The title to place at the top of panel 1. Default 'A-B'.
    addplabel     Adds a 'A:' or 'B:' to the title1 and title2. Default True.
    suptitle      The super-title to place at the top of the figure. Default ''.
    sector        Restrcit plot to a specific sector. Default 'None' (i.e. global).
    clim          A tuple of (min,max) color range OR a list of contour levels for the field plots. Default None.
    sigma         Sigma range for difference plot autocolor levels. Default is to span a 2. sigma range
    colormap      The name of the colormap to use for the field plots. Default None.
    extend        Can be one of 'both', 'neither', 'max', 'min'. Default None.
    centerlabels  If True, will move the colorbar labels to the middle of the interval. Default False.
    dlim          A tuple of (min,max) color range OR a list of contour levels for the difference plot. Default None.
    dcolormap     The name of the colormap to use for the difference plot. Default None.
    dextend       For the difference colorbar. Can be one of 'both', 'neither', 'max', 'min'. Default None.
    centerdlabels If True, will move the difference colorbar labels to the middle of the interval. Default False.
    nbins         The number of colors levels (used is clim is missing or only specifies the color range).
    landcolor     An rgb tuple to use for the color of land (no data). Default [.5,.5,.5].
    aspect        The aspect ratio of the figure, given as a tuple (W,H). Default [16,9].
    resolution    The vertical resolution of the figure given in pixels. Default 1280.
    axis          The axis handle to plot to. Default None.
    npanels       Number of panels to display (1, 2 or 3). Default 3.
    ignore        A value to use as no-data (NaN). Default None.
    save          Name of file to save figure in. Default None.
    debug         If true, report stuff for debugging. Default False.
    show          If true, causes the figure to appear on screen. Used for testing. Default False.
    webversion    If true, set options specific for displaying figures in a web browser. Default False.
    interactive   If true, adds interactive features such as zoom, close and cursor. Default False.
    """

    # set visual backend
    if interactive is False:
        plt.switch_backend("Agg")

    c = cm.dunne_pm()
    c = cm.dunne_rainbow()

    if (field1.shape) != (field2.shape):
        raise Exception("field1 and field2 must be the same shape")

    # Create coordinates if not provided
    xlabel, xunits, ylabel, yunits = formatting.createXYlabels(
        x, y, xlabel, xunits, ylabel, yunits
    )
    if debug:
        print("x,y label/units=", xlabel, xunits, ylabel, yunits)
    xCoord, yCoord = formatting.createXYcoords(field1, x, y)

    # Establish ranges for sectors
    lonRange, latRange, hspace, titleOffset = geoplot.sectorRanges(sector=sector)

    # Diagnose statistics
    if sector == None or sector == "global":
        if ignore is not None:
            maskedField1 = numpy.ma.masked_array(field1, mask=[field1 == ignore])
        else:
            maskedField1 = field1.copy()
        if ignore is not None:
            maskedField2 = numpy.ma.masked_array(field2, mask=[field2 == ignore])
        else:
            maskedField2 = field2.copy()
    else:
        maskedField1 = regionalMasking(field1, yCoord, xCoord, latRange, lonRange)
        maskedField2 = regionalMasking(field2, yCoord, xCoord, latRange, lonRange)
        areaCopy = numpy.ma.array(area, mask=maskedField1.mask, copy=True)
    s1Min, s1Max, s1Mean, s1Std, s1RMS = stats.calc(maskedField1, area, debug=debug)
    s2Min, s2Max, s2Mean, s2Std, s2RMS = stats.calc(maskedField2, area, debug=debug)
    dMin, dMax, dMean, dStd, dRMS = stats.calc(
        maskedField1 - maskedField2, area, debug=debug
    )
    if s1Mean is not None:
        dRxy = stats.corr(maskedField1 - s1Mean, maskedField2 - s2Mean, area)
    else:
        dRxy = None
    s12Min = min(s1Min, s2Min)
    s12Max = max(s1Max, s2Max)
    xLims = formatting.boundaryStats(xCoord)
    yLims = formatting.boundaryStats(yCoord)
    if debug:
        print("s1: min, max, mean =", s1Min, s1Max, s1Mean)
        print("s2: min, max, mean =", s2Min, s2Max, s2Mean)
        print("s12: min, max =", s12Min, s12Max)

    # Choose colormap
    if nbins is None and (clim is None or len(clim) == 2):
        cBins = 35
    else:
        cBins = nbins
    if nbins is None and (dlim is None or len(dlim) == 2):
        nbins = 35
    if colormap is None:
        colormap = chooseColorMap(s12Min, s12Max)
    cmap, norm, extend = cm.chooseColorLevels(
        s12Min, s12Max, colormap, clim=clim, nbins=cBins, extend=extend
    )

    def annotateStats(axis, sMin, sMax, sMean, sStd, sRMS, webversion=False):
        if webversion == True:
            fontsize = 9
        else:
            fontsize = 10
        axis.annotate(
            "max=%.5g\nmin=%.5g" % (sMax, sMin),
            xy=(0.0, 1.025),
            xycoords="axes fraction",
            verticalalignment="bottom",
            fontsize=fontsize,
        )
        if sMean is not None:
            axis.annotate(
                "mean=%.5g\nrms=%.5g" % (sMean, sRMS),
                xy=(1.0, 1.025),
                xycoords="axes fraction",
                verticalalignment="bottom",
                horizontalalignment="right",
                fontsize=fontsize,
            )
            axis.annotate(
                " sd=%.5g\n" % (sStd),
                xy=(1.0, 1.025),
                xycoords="axes fraction",
                verticalalignment="bottom",
                horizontalalignment="left",
                fontsize=fontsize,
            )

    if addplabel:
        preTitleA = "A: "
        preTitleB = "B: "
    else:
        preTitleA = ""
        preTitleB = ""

    if axis is None:
        formatting.setFigureSize(aspect, resolution, npanels=npanels, debug=debug)

    if npanels in [2, 3]:
        axis = plt.subplot(npanels, 1, 1)
        if sector == None or sector == "global":
            plt.pcolormesh(xCoord, yCoord, maskedField1, cmap=cmap, norm=norm)
            if interactive:
                addStatusBar(xCoord, yCoord, maskedField1)
            cb1 = plt.colorbar(fraction=0.08, pad=0.02, extend=extend)
            plt.xlim(xLims)
            plt.ylim(yLims)
            axis.axes.get_xaxis().set_visible(False)
        else:
            plotBasemapPanel(
                maskedField1,
                sector,
                xCoord,
                yCoord,
                lonRange,
                latRange,
                cmap,
                norm,
                interactive,
                extend,
            )
        if centerlabels and len(clim) > 2:
            cb1.set_ticks(0.5 * (clim[:-1] + clim[1:]))
        axis.set_facecolor(landcolor)
        annotateStats(axis, s1Min, s1Max, s1Mean, s1Std, s1RMS, webversion=webversion)
        if len(ylabel + yunits) > 0:
            plt.ylabel(formatting.label(ylabel, yunits))
        if len(title1) > 0:
            if webversion == True:
                axis.annotate(
                    preTitleA + title1,
                    xy=(0.5, 1.14),
                    xycoords="axes fraction",
                    verticalalignment="bottom",
                    horizontalalignment="center",
                    fontsize=12,
                )
            else:
                plt.title(preTitleA + title1)

        axis = plt.subplot(npanels, 1, 2)
        if sector == None or sector == "global":
            plt.pcolormesh(xCoord, yCoord, maskedField2, cmap=cmap, norm=norm)
            if interactive:
                addStatusBar(xCoord, yCoord, maskedField2)
            cb2 = plt.colorbar(fraction=0.08, pad=0.02, extend=extend)
            plt.xlim(xLims)
            plt.ylim(yLims)
            if npanels > 2:
                axis.axes.get_xaxis().set_visible(False)
        else:
            plotBasemapPanel(
                maskedField2,
                sector,
                xCoord,
                yCoord,
                lonRange,
                latRange,
                cmap,
                norm,
                interactive,
                extend,
            )
        if centerlabels and len(clim) > 2:
            cb2.set_ticks(0.5 * (clim[:-1] + clim[1:]))
        axis.set_facecolor(landcolor)
        annotateStats(axis, s2Min, s2Max, s2Mean, s2Std, s2RMS, webversion=webversion)
        if len(ylabel + yunits) > 0:
            plt.ylabel(formatting.label(ylabel, yunits))
        if len(title2) > 0:
            if webversion == True:
                axis.annotate(
                    preTitleB + title2,
                    xy=(0.5, titleOffset),
                    xycoords="axes fraction",
                    verticalalignment="bottom",
                    horizontalalignment="center",
                    fontsize=12,
                )
            else:
                plt.title(preTitleB + title2)

    if npanels in [1, 3]:
        axis = plt.subplot(npanels, 1, npanels)
        if sector == None or sector == "global":
            if dcolormap is None:
                dcolormap = chooseColorMap(dMin, dMax)
            if dlim is None and dStd > 0:
                cmap, norm, dextend = cm.chooseColorLevels(
                    dMean - sigma * dStd,
                    dMean + sigma * dStd,
                    dcolormap,
                    clim=dlim,
                    nbins=nbins,
                    extend="both",
                    autocenter=True,
                )
            else:
                cmap, norm, dextend = cm.chooseColorLevels(
                    dMin,
                    dMax,
                    dcolormap,
                    clim=dlim,
                    nbins=nbins,
                    extend=dextend,
                    autocenter=True,
                )
            plt.pcolormesh(
                xCoord, yCoord, maskedField1 - maskedField2, cmap=cmap, norm=norm
            )
            if interactive:
                addStatusBar(xCoord, yCoord, maskedField1 - maskedField2)
            if dextend is None:
                dextend = extend
            cb3 = plt.colorbar(fraction=0.08, pad=0.02, extend=dextend)  # was extend!
            if centerdlabels and len(dlim) > 2:
                cb3.set_ticks(0.5 * (dlim[:-1] + dlim[1:]))
            axis.set_facecolor(landcolor)
            plt.xlim(xLims)
            plt.ylim(yLims)
            annotateStats(axis, dMin, dMax, dMean, dStd, dRMS, webversion=webversion)
            if len(ylabel + yunits) > 0:
                plt.ylabel(formatting.label(ylabel, yunits))
            if len(title3) > 0:
                plt.title(title3)
        elif sector != None:
            # Copy data array, mask, and compute stats / color levels
            maskedDiffField = numpy.ma.array(maskedField1 - maskedField2)
            dMin, dMax, dMean, dStd, dRMS = stats.calc(
                maskedDiffField, areaCopy, debug=debug
            )
            if dcolormap is None:
                dcolormap = chooseColorMap(dMin, dMax, difference=True)
            if dlim is None and dStd > 0:
                cmap, norm, dextend = cm.chooseColorLevels(
                    dMean - sigma * dStd,
                    dMean + sigma * dStd,
                    dcolormap,
                    clim=dlim,
                    nbins=nbins,
                    extend="both",
                    autocenter=True,
                )
            else:
                cmap, norm, dextend = cm.chooseColorLevels(
                    dMin,
                    dMax,
                    dcolormap,
                    clim=dlim,
                    nbins=nbins,
                    extend=dextend,
                    autocenter=True,
                )
            # Set up Basemap projection
            plotBasemapPanel(
                maskedField1 - maskedField2,
                sector,
                xCoord,
                yCoord,
                lonRange,
                latRange,
                cmap,
                norm,
                interactive,
                dextend,
            )
            annotateStats(axis, dMin, dMax, dMean, dStd, dRMS, webversion=webversion)
            if len(ylabel + yunits) > 0:
                plt.ylabel(label(ylabel, yunits))
            if len(title3) > 0:
                if webversion == True:
                    axis.annotate(
                        title3,
                        xy=(0.5, titleOffset),
                        xycoords="axes fraction",
                        verticalalignment="bottom",
                        horizontalalignment="center",
                        fontsize=12,
                    )
                else:
                    plt.title(title3)
        else:
            raise ValueError("Invalid sector specified")

    if webversion:
        plt.subplots_adjust(hspace=hspace)
    if webversion == True:
        fig = plt.gcf()
        fig.text(
            0.5,
            0.02,
            "Generated by m6plot on dora.gfdl.noaa.gov",
            fontsize=9,
            horizontalalignment="center",
        )

    plt.suptitle(suptitle, y=1.0)

    if save is not None:
        plt.savefig(save, bbox_inches="tight")
    if interactive:
        addInteractiveCallbacks()
    if show:
        plt.show(block=False)

    return plt.gcf()
